<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AdvancedattrsHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class AdvancedattrsHelper
{
	/**
	 * getAdvancedAttrs
	 *
	 * @param   string  $extension_option  Params
	 * @param   int     $ref_id            Params
	 *
	 * @return	object
	 */
	public static function getAdvancedAttrs($extension_option, $ref_id)
	{
		$advanced_attrs = null;

		if (AUTOTWEETNG_JOOCIAL)
		{
			$advancedAttrsModel = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');
			$advancedAttrsModel->set('option-filter', $extension_option);
			$advancedAttrsModel->set('ref_id', $ref_id);
			$advancedAttrs = $advancedAttrsModel->getFirstItem();

			if ($advancedAttrs)
			{
				$advanced_attrs = json_decode($advancedAttrs->params);
			}
		}

		return $advanced_attrs;
	}

	/**
	 * getAdvancedAttrByReq
	 *
	 * @param   int  $req_id  Params
	 *
	 * @return	object
	 */
	public static function getAdvancedAttrByReq($req_id)
	{
		$advanced_attrs = null;

		if (AUTOTWEETNG_JOOCIAL)
		{
			$advancedAttrsModel = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');
			$advancedAttrsModel->set('request_id', $req_id);
			$advancedAttrs = $advancedAttrsModel->getFirstItem();

			if (($advancedAttrs) && ($advancedAttrs->request_id == $req_id))
			{
				$advanced_attrs = json_decode($advancedAttrs->params);
			}
		}

		return $advanced_attrs;
	}

	/**
	 * retrieveAdvancedAttrs
	 *
	 * @param   string  &$autotweet_advanced  Params
	 *
	 * @return	object
	 */
	public static function retrieveAdvancedAttrs(&$autotweet_advanced)
	{
		$autotweet_advanced = json_decode($autotweet_advanced);

		unset($autotweet_advanced->editorTitle);
		unset($autotweet_advanced->postthisLabel);
		unset($autotweet_advanced->evergreenLabel);
		unset($autotweet_advanced->agendaLabel);
		unset($autotweet_advanced->unix_mhdmdLabel);
		unset($autotweet_advanced->repeat_untilLabel);
		unset($autotweet_advanced->imageLabel);
		unset($autotweet_advanced->channelLabel);

		unset($autotweet_advanced->postthisDefaultLabel);
		unset($autotweet_advanced->postthisYesLabel);
		unset($autotweet_advanced->postthisNoLabel);

		unset($autotweet_advanced->descriptionLabel);
		unset($autotweet_advanced->hashtagsLabel);
		unset($autotweet_advanced->fulltextLabel);

		AutotweetBaseHelper::convertLocalUTCAgenda($autotweet_advanced->agenda);

		$advanced_attrs = $autotweet_advanced;

		list($isAdmin, $option, $controller, $task, $view, $layout, $articleid) = AutotweetBaseHelper::getControllerParams($autotweet_advanced);

		$isThirdPartyRequest = ( ($option == CAUTOTWEETNG)
				&& (
					// Other Joomla Extensions
					( ($view == 'requests') && (($task == 'save') || ($task == 'applyAjaxPluginAction')) )
				)
		);

		if (!$isThirdPartyRequest)
		{
			$advanced_attrs->client_id = $isAdmin;
			$advanced_attrs->option = $option;
			$advanced_attrs->controller = $controller;
			$advanced_attrs->task = $task;
			$advanced_attrs->view = $view;
			$advanced_attrs->layout = $layout;
			$advanced_attrs->ref_id = $articleid;
		}

		return $advanced_attrs;
	}

	/**
	 * saveAdvancedAttrs
	 *
	 * @param   object  &$advanced_attrs  Params
	 * @param   int     $articleid        Params
	 *
	 * @return	int
	 */
	public static function saveAdvancedAttrs(&$advanced_attrs, $articleid)
	{
		static $data = null;
		static $attr_id = null;

		$json_advanced_attrs = json_encode($advanced_attrs);
		$hash = md5($json_advanced_attrs);

		if ($data == $hash)
		{
			// What! Saved, but unidentifed ... flexicontent ...
			if (!isset($advanced_attrs->attr_id))
			{
				$advanced_attrs->attr_id = $attr_id;
			}

			return;
		}

		$data = $hash;
		$row = self::_getAdvancedattrs($advanced_attrs, $articleid);
		$id = $row->id;
		$row->reset();

		if ($id)
		{
			$row->load($id);

			if (!empty($row->params))
			{
				$params = json_decode($row->params);
				$advanced_attrs = (object) array_merge((array) $params, (array) $advanced_attrs);
				$json_advanced_attrs = json_encode($advanced_attrs);
				$hash = md5($json_advanced_attrs);
				$data = $hash;
			}
		}

		$advancedAttrs = array(
						'id' => $id,
						'client_id' => $advanced_attrs->client_id,
						'option' => $advanced_attrs->option,
						'controller' => $advanced_attrs->controller,
						'task' => $advanced_attrs->task,
						'view' => $advanced_attrs->view,
						'layout' => $advanced_attrs->layout,
						'ref_id' => $articleid,
						'params' => $json_advanced_attrs
		);

		$result = $row->save($advancedAttrs);

		if (!$id)
		{
			$id = $row->id;
		}

		$advanced_attrs->attr_id = $id;
		$attr_id = $id;

		$logger = AutotweetLogger::getInstance();

		if (!$result)
		{
			$logger->log(JLog::ERROR, 'advanced_attrs: error storing message to database advanced_attrs, article id = ' . $articleid . ', error message = ' . $row->getError());
		}
		else
		{
			$logger->log(JLog::INFO, 'Saved advanced_attrs', $advancedAttrs);
		}

		return $id;
	}

	/**
	 * _getAdvancedattrs
	 *
	 * @param   object  &$advanced_attrs  Params
	 * @param   int     $articleid        Params
	 *
	 * @return	object
	 */
	private static function _getAdvancedattrs(&$advanced_attrs, $articleid)
	{
		$advancedAttrsModel = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');

		if (isset($advanced_attrs->option))
		{
			$advancedAttrsModel->set('option-filter', $advanced_attrs->option);
		}
		elseif (isset($advanced_attrs->option_filter))
		{
			$advancedAttrsModel->set('option-filter', $advanced_attrs->option_filter);
		}

		unset($advanced_attrs->option_filter);

		$advancedAttrsModel->set('ref_id', $articleid);
		$row = $advancedAttrsModel->getFirstItem();

		return $row;
	}

	/**
	 * assignRequestId
	 *
	 * @param   int  $attr_id  Params
	 * @param   int  $req_id   Params
	 *
	 * @return	int
	 */
	public static function assignRequestId($attr_id, $req_id)
	{
		$row = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel')->getTable();
		$row->load($attr_id);
		$data = array(
			'id' => $attr_id,
			'request_id' => $req_id
		);
		$row->save($data);
	}

	/**
	 * execute
	 *
	 * @param   int    $req_id  Params
	 * @param   array  &$data   Params
	 *
	 * @return	void
	 */
	public static function execute($req_id,  &$data)
	{
		// Advanced Attrs - Initialization
		$advancedAttrsModel = F0FModel::getTmpInstance('Advancedattrs', 'AutoTweetModel');
		$advancedAttrsModel->set('request_id', $req_id);
		$advancedAttrs = $advancedAttrsModel->getFirstItem();

		if (!$advancedAttrs)
		{
			return;
		}

		$params = json_decode($advancedAttrs->params);

		// No params
		if (!$params)
		{
			return;
		}

		// Joocial AND Params

		// Checking if there are future tasks
		if ((isset($params->agenda)) && (count($params->agenda) > 0))
		{
			$publish_up = self::getNextAgendaDate($params->agenda);

			if ($publish_up)
			{
				$data['publish_up'] = $publish_up;

				// Not finished yet, we have at least a date to schedule
				$data['published'] = false;
			}
			else
			{
				$data['published'] = true;
			}
		}

		// Repeat
		if ((isset($params->unix_mhdmd)) && (!empty($params->unix_mhdmd)))
		{
			$repeat_until = null;

			if (isset($params->repeat_until))
			{
				$repeat_until = $params->repeat_until;
			}

			// Not finished yet, we have at least a date to schedule
			$nextRepeatDate = self::_getNextRepeatPublishUp($params->unix_mhdmd, $repeat_until);

			if ($nextRepeatDate)
			{
				$data['publish_up'] = $nextRepeatDate->toSql();
				$data['published'] = false;
			}
			else
			{
				$data['published'] = true;
			}
		}

		$params = json_encode($params);
		$advancedattr_data = array('params' => $params);
		$advancedAttrs->save($advancedattr_data);
	}

	/**
	 * getNextAgendaDate
	 *
	 * @param   array  $agenda  Param
	 *
	 * @return	string
	 */
	public static function getNextAgendaDate($agenda)
	{
		$n = count($agenda);

		if ($n == 0)
		{
			return null;
		}

		$now = JFactory::getDate();
		$now_unix = $now->toUnix();

		// The first date

		$i = 0;

		do
		{
			$publish_up = $agenda[$i];
			$publish_up_unix = JFactory::getDate($publish_up)->toUnix();
			$i++;
		}

		while (($i < $n) && ($publish_up_unix < $now_unix));

		if ($publish_up_unix > $now_unix)
		{
			return $publish_up;
		}

		return null;
	}

	/**
	 * _getNextRepeatPublishUp
	 *
	 * @param   string  $repeat        Param
	 * @param   date    $repeat_until  Param
	 *
	 * @return	string
	 */
	private static function _getNextRepeatPublishUp($repeat, $repeat_until)
	{
		$dates = self::getRepeatPublishUpDates($repeat, $repeat_until, 1);

		if (count($dates) == 1)
		{
			return $dates[0];
		}

		return null;
	}

	/**
	 * getRepeatPublishUpDates
	 *
	 * @param   string  $repeat        Param
	 * @param   date    $repeat_until  Param
	 * @param   int     $n             Param
	 *
	 * @return	string
	 */
	public static function getRepeatPublishUpDates($repeat, $repeat_until, $n = 30)
	{
		$dates = array();

		try
		{
			$current_date = JFactory::getDate();
			$dpcheck = EParameter::getComponentParam(CAUTOTWEETNG, 'dpcheck_time_intval', 12) * 3600;
			$current_unix = $current_date->toUnix() + $dpcheck;

			$datePublishUp = new DateTime;
			$i = 0;

			while ($i < $n)
			{
				$datePublishUp->setTimestamp($current_unix);

				$publish_up = TextUtil::nextScheduledDate($repeat, $datePublishUp);

				if ( ($publish_up) && (!empty($repeat_until))
					&& ($publish_up->toUnix() > JFactory::getDate($repeat_until)->toUnix()) )
				{
					break;
				}

				$dates[] = $publish_up;
				$current_date = JFactory::getDate($publish_up);
				$current_unix = $current_date->toUnix();
				$i++;
			}
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, 'getRepeatPublishUpDates' . $e->getMessage());
		}

		return $dates;
	}

	/**
	 * getEditLink
	 *
	 * @param   string  $client_id  Param
	 * @param   string  $option     Param
	 * @param   string  $ref_id     Param
	 * @param   string  $req_id     Param
	 *
	 * @return	string
	 */
	public static function getEditLink($client_id, $option, $ref_id, $req_id)
	{
		$link = null;

		switch ($option)
		{
			case 'com_content':
				$link = 'index.php?option=com_content&task=article.edit&id=' . $ref_id;
				break;
			case 'com_autotweet':
				$link = 'index.php?option=com_autotweet&view=composer&req-id=' . $req_id;
				break;
			case 'com_k2':
				$link = 'index.php?option=com_k2&view=item&cid=' . $ref_id;
				break;
			case 'com_easyblog':
				$link = 'index.php?option=com_easyblog&c=blogs&task=edit&blogid=' . $ref_id;
				break;
			case 'com_flexicontent':
				$link = 'index.php?option=com_flexicontent&task=items.edit&cid[]=' . $ref_id;
				break;
			case 'com_zoo':
				$link = 'index.php?option=com_zoo&controller=item&task=edit&cid[]=' . $ref_id;
				break;
		}

		return $link;
	}
}
