<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Blogger, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweet Blogger channel.
**/

JLoader::import('channel', dirname(__FILE__));

JLoader::register('Google_Client', dirname(__FILE__) . '/google/Google_Client.php');
JLoader::register('Google_ServiceResource', dirname(__FILE__) . '/google/service/Google_ServiceResource.php');
JLoader::register('Google_BloggerService', dirname(__FILE__) . '/google/contrib/Google_BloggerService.php');

/**
 * BloggerChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class BloggerChannelHelper extends ChannelHelper
{
	protected $bloggerClient = null;

	protected $blogger = null;

	protected $client_id = null;

	protected $client_secret = null;

	protected $developer_key = null;

	protected $access_token = null;

	protected $is_auth = null;

	protected $me = null;

	protected $blogs = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel  Params.
	 */
	public function __construct($channel)
	{
		parent::__construct($channel);

		if ($channel->id)
		{
			$this->client_id = $this->channel->params->get('client_id');
			$this->client_secret = $this->channel->params->get('client_secret');
			$this->developer_key = $this->channel->params->get('developer_key');

			$access_token = $this->channel->params->get('access_token');
			$this->setAccessToken($access_token);
		}
	}

	/**
	 * setAccessToken
	 *
	 * @param   string  $access_token  Param
	 *
	 * @return	void
	 */
	public function setAccessToken($access_token)
	{
		$this->access_token = $access_token;
	}

	/**
	 * getAccessToken
	 *
	 * @return	string
	 */
	public function getAccessToken()
	{
		return $this->access_token;
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->bloggerClient)
		{
			JLoader::load('Google_Client');
			JLoader::load('Google_ServiceResource');
			JLoader::load('Google_BloggerService');

			$this->bloggerClient = new google_api\Google_Client;

			$sitename = JFactory::getConfig()->get('sitename');
			$this->bloggerClient->setApplicationName($sitename);

			$this->bloggerClient->setClientId($this->client_id);
			$this->bloggerClient->setClientSecret($this->client_secret);
			$this->bloggerClient->setDeveloperKey($this->developer_key);

			/*
			$requestVisibleActions = array(
				'http://schemas.google.com/AddActivity'
			);
			$this->bloggerClient->setRequestVisibleActions($requestVisibleActions);
			*/

			if ((isset($this->channel->id)) && ($this->channel->id))
			{
				require_once dirname(__FILE__) . '/../../controllers/bloggerchannels.php';

				$url = AutotweetControllerBloggerChannels::getCallbackUrl($this->channel->id);
				$this->bloggerClient->setRedirectUri($url);
			}

			$access_token = $this->getAccessToken();

			if (!empty($access_token))
			{
				$this->bloggerClient->setAccessToken($access_token);
			}

			$this->blogger = new google_api\Google_BloggerService($this->bloggerClient);
		}

		return $this->bloggerClient;
	}

	/**
	 * isAuth()
	 *
	 * @return	bool
	 */
	public function isAuth()
	{
		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');
		$access_token = $this->getAccessToken();

		if (empty($access_token))
		{
			$this->access_token = null;

			return false;
		}

		try
		{
			$this->getApiInstance();

			// First try
			$isExpired = $this->bloggerClient->isAccessTokenExpired();

			if (!$isExpired)
			{
				$user = $this->getUser();

				return true;
			}

			$this->_refreshToken();

			// Second try, and the last
			$isExpired = $this->bloggerClient->isAccessTokenExpired();

			if ($isExpired)
			{
				// Invalidating access_token
				$ch->setToken($this->channel->id, 'access_token', '');
			}
			else
			{
				$user = $this->getUser();

				// We Ok, and it's new one!
				$ch->setToken($this->channel->id, 'access_token', $this->getAccessToken());

				return true;
			}
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
		}

		return false;
	}

	/**
	 * _refreshToken()
	 *
	 * @return	void
	 */
	private function _refreshToken()
	{
		$this->access_token = $this->bloggerClient->getAccessToken();
		$access_token = json_decode($this->access_token);

		$this->bloggerClient->refreshToken($access_token->refresh_token);

		$this->access_token = $this->bloggerClient->getAccessToken();
	}

	/**
	 * authenticate
	 *
	 * @param   string  $code  Param
	 *
	 * @return	bool
	 */
	public function authenticate($code)
	{
		$this->getApiInstance();

		$this->access_token = $this->bloggerClient->authenticate($code);

		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');

		if ($this->access_token)
		{
			$ch->setToken($this->channel->id, 'access_token', $this->access_token);

			return true;
		}
		else
		{
			JFactory::getApplication()->enqueueMessage('Unable to retrieve access token.', 'error');

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
		}

		return false;
	}

	/**
	 * getAuthorizationUrl
	 *
	 * @return	string
	 */
	public function getAuthorizationUrl()
	{
		$this->getApiInstance();

		return $this->bloggerClient->createAuthUrl();
	}

	/**
	 * getUser
	 *
	 * @return	object
	 */
	public function getUser()
	{
		if (!$this->me)
		{
			$this->me = $this->blogger->users->get('self');
		}

		return $this->me;
	}

	/**
	 * getUser
	 *
	 * @return	object
	 */
	public function getBlogs()
	{
		if (!$this->blogs)
		{
			$this->blogs = $this->blogger->blogs->listByUser('self');
		}

		return $this->blogs;
	}

	/**
	 * getExpiresIn
	 *
	 * @return	string
	 */
	public function getExpiresIn()
	{
		if (($this->access_token) && (!empty($this->access_token)))
		{
			$access_token = json_decode($this->access_token);

			$created = $access_token->created;
			$expires_in = $access_token->expires_in;

			$expires_in += $created;

			return JHtml::_('date', $expires_in, JText::_('COM_AUTOTWEET_DATE_FORMAT'));
		}

		return null;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		$isAuth = $this->isAuth();

		if (!$isAuth)
		{
			return array(
				false,
				JText::_('COM_AUTOTWEET_CHANNEL_BLOGGER_NOT_AUTH_ERR')
			);
		}

		$result = array(false, 'Blogger Unknown', null);

		try
		{
			$image_url = $data->image_url;
			$media_mode = $this->getMediaMode();
			$org_url = $data->org_url;
			$content = $this->renderPost($this->channel->id, 'bloggerchannel', $message, $data);

			$blogid = $this->get('blogid');
			$blog = new google_api\Google_PostBlog;
			$blog->setId($blogid);

			$googlePost = new google_api\Google_Post;
			$googlePost->setKind('blogger#post');
			$googlePost->setBlog($blog);
			$googlePost->setTitle($data->title);
			$googlePost->setContent($content);
			$googlePost->setUrl($org_url);

			$googlePost = $this->blogger->posts->insert($blogid, $googlePost);

			$msg = 'Post id: ' . $googlePost['id'] . ' - ' . $googlePost['url'];
			$result = array(
				true,
				$msg
			);
		}
		catch (Exception $e)
		{
			return array(
							false,
							$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * hasWeight
	 *
	 * @return	bool
	 */
	public function hasWeight()
	{
		return true;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}
}
