<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Scoopit, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweet EasySocial channel.
**/

JLoader::import('channel', dirname(__FILE__));

include_once JPATH_ROOT . '/administrator/components/com_easysocial/includes/foundry.php';

/**
 * EasySocialChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class EasySocialChannelHelper extends ChannelHelper
{
	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		try
		{
			$media_mode = $this->getMediaMode();

			$image_url = null;

			if ($media_mode != 'message')
			{
				$image_url = $data->image_url;
			}

			$author = $data->xtform->get('author');
			$userId = JUserHelper::getUserId($author);

			if (!class_exists('FD'))
			{
				throw new Exception('EasySocial is not installed!');
			}

			// Get the current view.
			$my 	= FD::user($userId);

			// Load our story library
			$story 	= FD::story(SOCIAL_TYPE_USER);

			// Posted data.
			$post = array();
			$post['content'] = $message;

			$post['attachment'] = 'story';

			if ($image_url)
			{
				$post['attachment'] = 'links';
				$post['links_title'] = $data->title;
				$post['links_description'] = $data->fulltext;
				$post['links_url'] = $data->url;
				$post['links_image'] = $image_url;
			}

			$post['target'] = $userId;
			$post['cluster'] = '';
			$post['clusterType'] = '';
			$post['privacy'] = 'public';
			$post['privacyCustom'] = '';

			// Check if the user being viewed the same user or other user.
			$id 		= $post[ 'target' ];
			$targetId	= ( $my->id != $id ) ? $id : '';

			// Determine the post types.
			$type 		= isset($post['attachment']) && !empty($post['attachment']) ? $post['attachment'] : SOCIAL_TYPE_STORY;

			// Check if the content is empty only for story based items.
			if ((!isset($post['content']) || empty($post['content'])) && $type == SOCIAL_TYPE_STORY)
			{
				throw new Exception(JText::_('COM_EASYSOCIAL_STORY_PLEASE_POST_MESSAGE'));
			}

			// Check if the content is empty and there's no photos.
			if ((!isset($post[ 'photos' ]) || empty($post[ 'photos'])) && $type == 'photos')
			{
				throw new Exception(JText::_('COM_EASYSOCIAL_STORY_PLEASE_ADD_PHOTO'));
			}

			// We need to allow raw because we want to allow <,> in the text but it should be escaped during display
			$content = $message;

			// Check whether the user can really post something on the target
			if ($targetId)
			{
				$privacy = FD::privacy($my->id);
				$state = $privacy->validate('profiles.post.status', $targetId, SOCIAL_TYPE_USER);

				if (!$state)
				{
					throw new Exception(JText::_('COM_EASYSOCIAL_STORY_NOT_ALLOW_TO_POST'));
				}
			}

			$location = null;
			$friends = array();
			$contextIds = 0;

			// For photos that are posted on the story form
			if ($type == 'photos' && isset($post['photos']))
			{
				$contextIds = $post['photos'];
			}

			// Check if there are mentions provided from the post.
			$mentions = isset($post['mentions']) ? $post['mentions'] : array();

			// Process moods here
			$mood = FD::table('Mood');
			$hasMood = $mood->bindPost($post);

			// Set the privacy for the album
			$privacy = $post['privacy'];
			$customPrivacy = $post['privacyCustom'];
			$privacyRule = ( $type == 'photos' ) ? 'photos.view' : 'story.view';

			$cluster = isset( $post[ 'cluster' ]  ) ? $post[ 'cluster' ] : '';
			$clusterType = isset( $post[ 'clusterType' ]  ) ? $post[ 'clusterType' ] : '';
			$isCluster = ($cluster) ? true : false;

			$args = array(
				'content' => $content,
				'contextIds' => $contextIds,
				'contextType' => $type,
				'actorId' => $my->id,
				'targetId' => $targetId,
				'location' => $location,
				'with' => $friends,
				'mentions' => $mentions,
				'cluster' => $cluster,
				'clusterType' => $clusterType,
				'mood' => $mood,
				'privacyRule' => $privacyRule,
				'privacyValue' => $privacy,
				'privacyCustom' => $customPrivacy
			);

			// Create the stream item
			$stream		= $story->create($args);

			if ($image_url)
			{
				$this->_storeLinks($stream, $post['links_url'], $post['links_title'], $post['links_description'], $post['links_image']);
			}

			// @badge: story.create
			// Add badge for the author when a report is created.
			$badge = FD::badges();
			$badge->log('com_easysocial', 'story.create', $my->id, JText::_('COM_EASYSOCIAL_STORY_BADGE_CREATED_STORY'));

			// @points: story.create
			// Add points for the author when a report is created.
			$points = FD::points();
			$points->assign('story.create', 'com_easysocial', $my->id);

			$result = array(
				true,
				'OK - ' . $stream->getPermalink(false, true)
			);
		}
		catch (Exception $e)
		{
			return array(
							false,
							$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * Stores any link assets
	 *
	 * @param   object  $streamItem  Param
	 * @param   string  $link        Param
	 * @param   string  $title       Param
	 * @param   string  $content     Param
	 * @param   string  $image       Param
	 *
	 * @return	bool
	 */
	private function _storeLinks($streamItem, $link, $title, $content, $image)
	{
		// If there's no data, we don't need to store in the assets table.
		if (empty($title) && empty($content) && empty($image))
		{
			return false;
		}

		// Cache the image if necessary
		$links = FD::links();
		$uri = $links->cache($image);

		$registry = FD::registry();
		$registry->set('title', $title);
		$registry->set('content', $content);
		$registry->set('image', $image);
		$registry->set('link', $link);
		$registry->set('cached', false);

		// Image link should only be modified when the file exists
		if ($uri !== false)
		{
			$registry->set('cached', true);
			$registry->set('image', $uri);
		}

		// Store the link object into the assets table
		$assets = FD::table('StreamAsset');
		$assets->stream_id = $streamItem->uid;
		$assets->type = 'links';
		$assets->data = $registry->toString();

		// Store the assets
		$state = $assets->store();

		return $state;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}
}
