<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Xing, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweet Xing channel.
**/

JLoader::import('channel', dirname(__FILE__));
require_once dirname(__FILE__) . '/OAuth/OAuth.php';
JLoader::register('XingOAuth', dirname(__FILE__) . '/xingoauth/xingoauth.php');

/**
 * XingChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class XingChannelHelper extends ChannelHelper
{
	protected $xingClient = null;

	protected $consumer_key = null;

	protected $consumer_secret = null;

	protected $access_token = null;

	protected $access_secret = null;

	protected $is_auth = null;

	protected $me = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel  Params.
	 */
	public function __construct($channel)
	{
		parent::__construct($channel);

		if ($channel->id)
		{
			$this->consumer_key = $this->channel->params->get('consumer_key');
			$this->consumer_secret = $this->channel->params->get('consumer_secret');
			$this->access_token = $this->channel->params->get('access_token');
			$this->access_secret = $this->channel->params->get('access_secret');
		}
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->xingClient)
		{
			JLoader::load('XingOAuth');
			$this->xingClient = new xingoauth\XingOAuth(
					$this->consumer_key,
					$this->consumer_secret,
					$this->access_token,
					$this->access_secret
			);
		}

		return $this->xingClient;
	}

	/**
	 * isAuth()
	 *
	 * @return	mixed
	 */
	public function isAuth()
	{
		if (empty($this->access_token))
		{
			$this->access_token = null;

			return false;
		}

		try
		{
			$this->getApiInstance();

			$return = $this->xingClient->get('users/me');

			if (!isset($return->users))
			{
				if (isset($return->error_name))
				{
					throw new Exception($return->error_name . ' - ' . $return->message);
				}

				return false;
			}

			return $return;
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
		}

		return false;
	}

	/**
	 * authenticate
	 *
	 * @param   string  $oauth_token     Param
	 * @param   string  $oauth_verifier  Param
	 *
	 * @return	bool
	 */
	public function authenticate($oauth_token, $oauth_verifier)
	{
		$this->getApiInstance();
		$access_token = $this->xingClient->getAccessToken($oauth_verifier);

		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');

		if (($access_token) && (is_array($access_token)) && (array_key_exists('oauth_token_secret', $access_token)))
		{
			$ch->setToken($this->channel->id, 'access_token', $access_token['oauth_token']);
			$ch->setToken($this->channel->id, 'access_secret', $access_token['oauth_token_secret']);
			$ch->setToken($this->channel->id, 'user_id', $access_token['user_id']);

			return true;
		}
		else
		{
			JFactory::getApplication()->enqueueMessage('Unable to retrieve access token.', 'error');

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
			$ch->setToken($this->channel->id, 'access_secret', '');
			$ch->setToken($this->channel->id, 'user_id', '');
		}

		return false;
	}

	/**
	 * getAuthorizationUrl
	 *
	 * @return	string
	 */
	public function getAuthorizationUrl()
	{
		require_once dirname(__FILE__) . '/../../controllers/xingchannels.php';
		$this->getApiInstance();

		$redirectUri = AutotweetControllerXingChannels::getCallbackUrl($this->channel->id);
		$request_token = $this->xingClient->getRequestToken($redirectUri);

		// Invalid OAuth signature - One more time
		if (count($request_token) != 3)
		{
			$request_token = $this->xingClient->getRequestToken($redirectUri);

			// No more tries
			if (count($request_token) != 3)
			{
				throw new Exception('Invalid OAuth signature');
			}
		}

		$url = $this->xingClient->getAuthorizeURL($request_token);

		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');
		$ch->setToken($this->channel->id, 'access_token', $request_token['oauth_token']);
		$ch->setToken($this->channel->id, 'access_secret', $request_token['oauth_token_secret']);

		return $url;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		$logger = AutotweetLogger::getInstance();
		$logger->log(JLog::INFO, 'sendXingMessage', $message);

		$isAuth = $this->isAuth();

		if (!$isAuth)
		{
			return array(
				false,
				JText::_('COM_AUTOTWEET_CHANNEL_XING_NOT_AUTH_ERR')
			);
		}

		$result = array(false, 'Xing Unknown', null);

		try
		{
			$this->getApiInstance();

			$parameters = array();

			$url = $data->org_url;

			if (empty($url))
			{
				$url = RouteHelp::getInstance()->getRoot();
			}

			$parameters['uri'] = $url;
			$parameters['text'] = $message;

			$post = $this->xingClient->post('users/me/share/link', $parameters);

			if ((isset($post->created_at)) && (isset($post->ids)) && (count($post->ids) == 1))
			{
				$id = $post->ids[0];
				$parts = explode('_', $id);
				$story = $parts[0];

				$result = array(
					true
				);
				$result[] = 'OK - https://www.xing.com/feedy/stories/' . $story;

				/*
				$ret = $this->xingClient->post('activities/' . $id . '/share');

				if (isset($ret->error_name))
				{
					$logger = AutotweetLogger::getInstance();
					$logger->log(JLog::ERROR, 'Xing sendMessage ' . $ret->error_name . ' - ' . $ret->message);
				}
				*/
			}
			else
			{
				$result = array(
					false
				);
				$result[] = $post->error_name . ' - ' . $post->message;
			}
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}

	/**
	 * getSocialUrl
	 *
	 * @param   object  $user  Param
	 *
	 * @return	string
	 */
	public function getSocialUrl($user)
	{
		return $user->permalink;
	}
}
