<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutotweetModelFeedK2
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class AutotweetModelFeedK2 extends F0FModel
{
	/**
	 * Binds the data to the model and tries to save it
	 *
	 * @param   array|object  $data  The source data array or object
	 *
	 * @return  boolean  True on success
	 */
	public function save($data)
	{
		$post = array();

		$post['id'] = null;

		$post['title'] = $data['title'];
		$post['alias'] = $data['alias'];
		$post['catid'] = $data['catid'];
		$post['featured'] = $data['featured'];
		$post['published'] = $data['state'];
		$post['text'] = FeedTextHelper::joinArticleText($data['introtext'], $data['fulltext']);
		$post['language'] = $data['language'];
		$post['created_by'] = $data['created_by'];
		$post['created_by_alias'] = $data['created_by_alias'];
		$post['access'] = $data['access'];
		$post['created'] = $data['created'];
		$post['publish_up'] = $data['publish_up'];
		$post['publish_down'] = $data['publish_down'];
		$post['metadesc'] = $data['metadesc'];
		$post['metakey'] = $data['metakey'];

		return $this->_save($post);
	}

	/**
	 * _save
	 *
	 * @param   F0FTable  &$post  Param
	 *
	 * @return  bool
	 */
	protected function _save(&$post)
	{
		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_k2/tables');
		JLoader::register('K2Table', JPATH_ADMINISTRATOR . '/components/com_k2/tables/table.php');

		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.archive');

		$db = JFactory::getDBO();
		$row = JTable::getInstance('K2Item', 'Table');
		$params = JComponentHelper::getParams('com_k2');
		$nullDate = $db->getNullDate();

		if (!$row->bind($post))
		{
			return false;
		}

		$isNew = true;

		// MergeEditors
		$text = $post['text'];

		// XssFiltering
		$a = array();
		$filter = new JFilterInput($a, $a, 1, 1, 0);
		$text = $filter->clean($text);
		list($row->introtext, $row->fulltext) = FeedTextHelper::splitArticleText($text);
		$row->ordering = $row->getNextOrder("catid = {$row->catid} AND trash = 0");

		if ($row->featured)
		{
			$row->featured_ordering = $row->getNextOrder("featured = 1 AND trash = 0", 'featured_ordering');
		}

		if ($row->created && strlen(trim($row->created)) <= 10)
		{
			$row->created .= ' 00:00:00';
		}

		$config = JFactory::getConfig();
		$tzoffset = K2_JVERSION == '30' ? $config->get('offset') : $config->getValue('config.offset');

		$date = JFactory::getDate($row->created, $tzoffset);
		$row->created = $date->toSql();

		if (strlen(trim($row->publish_up)) <= 10)
		{
			$row->publish_up .= ' 00:00:00';
		}

		$date = JFactory::getDate($row->publish_up, $tzoffset);
		$row->publish_up = $date->toSql();

		$row->publish_down = $nullDate;

		if (!$row->check())
		{
			return false;
		}

		$dispatcher = JDispatcher::getInstance();
		JPluginHelper::importPlugin('k2');
		$result = $dispatcher->trigger('onBeforeK2Save', array(
			&$row,
			$isNew
			)
		);

		if (in_array(false, $result, true))
		{
			return false;
		}

		// Trigger the finder before save event
		$dispatcher = JDispatcher::getInstance();
		JPluginHelper::importPlugin('finder');
		$results = $dispatcher->trigger('onFinderBeforeSave', array(
			'com_k2.item',
			$row,
			$isNew
			)
		);

		if (!$row->store())
		{
			return false;
		}

		$this->id = $db->insertid();

		// !DisableCompactOrdering
		$row->reorder("catid = {$row->catid} AND trash = 0");

		// !DisableCompactOrdering
		if ($row->featured)
		{
			$row->reorder("featured = 1 AND trash = 0", 'featured_ordering');
		}

		$row->checkin();

		$cache = JFactory::getCache('com_k2');
		$cache->clean();

		$dispatcher->trigger('onAfterK2Save', array(
				&$row,
				$isNew
			)
		);

		JPluginHelper::importPlugin('content');
		$dispatcher->trigger('onContentAfterSave', array(
				'com_k2.item',
				&$row,
				$isNew
			)
		);

		// Trigger the finder after save event
		$dispatcher = JDispatcher::getInstance();
		JPluginHelper::importPlugin('finder');
		$results = $dispatcher->trigger('onFinderAfterSave', array(
				'com_k2.item',
				$row,
				$isNew
			)
		);

		return true;
	}
}
