<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutotweetModelSessions
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class AutotweetModelSessions extends F0FModel
{
	/**
	 * buildQuery
	 *
	 * @param   bool  $overrideLimits  Param
	 *
	 * @return	F0FQuery
	 */
	public function buildQuery($overrideLimits = false)
	{
		$db = $this->getDbo();

		$query = F0FQueryAbstract::getNew($db)->select('*')->from($db->quoteName('#__autotweet_sessions'));

		$fltOAuthKey = $this->getState('oauth_key', null, 'string');
		$query->where($db->qn('oauth_key') . ' = ' . $db->q($fltOAuthKey));

		$fltOAuthType = $this->getState('oauth_type', null, 'string');
		$query->where($db->qn('oauth_type') . ' = ' . $db->q($fltOAuthType));

		$expired = $this->getExpiredDate();

		// Valid session
		$query->where(
			'(' . $db->qn('created') . ' > ' . $db->q($expired) . ' OR ' .
			$db->qn('modified') . ' > ' . $db->q($expired) . ')'
		);

		return $query;
	}

	/**
	 * getExpiredDate
	 *
	 * @return	string
	 */
	private function getExpiredDate()
	{
		// 24hs * 60min
		$lifetime = JFactory::getConfig()->get('mapis_session_lifetime', 1440);

		$expiration = time() - $lifetime * 60;
		$expired = JFactory::getDate($expiration)->toSql();

		return $expired;
	}

	/**
	 * purge
	 *
	 * @return	void
	 */
	public function purge()
	{
		$db = $this->getDbo();
		$query = F0FQueryAbstract::getNew($db)->delete($db->quoteName('#__autotweet_sessions'));
		$expired = $this->getExpiredDate();
		$query->where(
			'(' . $db->qn('created') . ' < ' . $db->q($expired) . ' AND ' .
				$db->qn('modified') . ' = ' . $db->q('0000-00-00 00:00:00') . ')'
			. ' OR ' .
			'(' . $db->qn('modified') . ' < ' . $db->q($expired) . ' AND ' .
				$db->qn('modified') . ' <> ' . $db->q('0000-00-00 00:00:00') . ')'
		);
		$db->setQuery($query);
		$db->execute();
	}
}
