<?php
/**
 * @version     2.1.0
 * @package     com_cloudbackup
 * @copyright   Copyright (C) 2012 - 2013. Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://bhartiy.com
 *
 * CloudBackup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * CloudBackup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CloudBackup; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
**/

// No direct access.
defined('_JEXEC') or die;

jimport('joomla.application.component.controlleradmin');

/**
 * Backups list controller class.
 */
class CloudbackupControllerBackups extends JControllerAdmin
{
	/**
	 * Proxy for getModel.
	 * @since		1.6
	 */
	public function getModel($name = '', $prefix = 'CloudbackupModel')
	{
		$model = parent::getModel($name, $prefix, array('ignore_request' => true));
		return $model;
	}
	
	public function getGoogleFile()
	{
		$fileId = JRequest::getVar('fileid', null);
		if(empty($fileId)) {
			JError::raiseWarning(505, JText::_('COM_CLOUDBACKUP_DOWNLOAD_NO_FILE_SELECTED'));
			return false;
		}

		$service = CloudbackupHelper::getDriveService();
		
		try {
			$file = $service->files->get($fileId);
		} catch (Exception $e) {
			JError::raiseWarning(505, JText::sprintf('COM_CLOUDBACKUP_DOWNLOAD_ERROR', $e->getMessage()));
			$this->setRedirect(JRoute::_('index.php?option=com_cloudbackup', false));
			return false;
		}
		try {
			$content = null;
			$link = $file['downloadUrl'];
			if ($link) {
				$request = new apiHttpRequest($link, 'GET', null, null);
				$httpRequest = apiClient::$io->authenticatedRequest($request);
				if ($httpRequest->getResponseHttpCode() == 200) {
					$content = $httpRequest->getResponseBody();
				}
			} else {
				JError::raiseWarning(505, JText::sprintf('COM_CLOUDBACKUP_DOWNLOAD_FILE_NOT_FOUND', $e->getMessage()));
				$this->setRedirect(JRoute::_('index.php?option=com_cloudbackup', false));
				return false;
			}
		} catch (Exception $e) {
			JError::raiseWarning(505, JText::sprintf('COM_CLOUDBACKUP_DOWNLOAD_ERROR', $e->getMessage()));
			$this->setRedirect(JRoute::_('index.php?option=com_cloudbackup', false));
			return false;
		}
		
		if(empty($content)) {
			JError::raiseWarning(505, JText::sprintf('COM_CLOUDBACKUP_DOWNLOAD_ERROR'));
			$this->setRedirect(JRoute::_('index.php?option=com_cloudbackup', false));
			return false;
		}
		
		header('Cache-Control: private');
		header('Content-Description: '.$file['description']);
		header('Content-Disposition: attachment; filename='.$file['title']);
		header('Content-Type: '.$file['mimeType']);
		header('Content-Transfer-Encoding: binary');
		echo $content;
		jexit();
	}

	public function configGDrive()
	{
		$app = JFactory::getApplication();
		$app->redirect(JRoute::_('index.php?option=com_cloudbackup&task=config.edit&id=1', false));
		return;
	}
	
	public function configDropbox()
	{
		$app = JFactory::getApplication();
		$app->redirect(JRoute::_('index.php?option=com_cloudbackup&task=config.edit&id=2', false));
		return;
	}
	
}