<?php
/**
 * @version     2.1.0
 * @package     com_cloudbackup
 * @copyright   Copyright (C) 2012 - 2013. Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://bhartiy.com
 *
 * CloudBackup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * CloudBackup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CloudBackup; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
**/

defined('_JEXEC') or die;

jimport('joomla.application.component.modellist');

/**
 * Methods supporting a list of Cloudbackup records.
 */
class CloudbackupModelprofiles extends JModelList
{

    /**
     * Constructor.
     *
     * @param    array    An optional associative array of configuration settings.
     * @see        JController
     * @since    1.6
     */
    public function __construct($config = array())
    {
        if (empty($config['filter_fields'])) {
            $config['filter_fields'] = array(
                                'id', 'a.id',
                'interval', 'a.interval',
                'backupfolder', 'a.backupfolder',
                'backuptype', 'a.backuptype',
                'excludefiles', 'a.excludefiles',
                'excludetables', 'a.excludetables',
                'lastattempt', 'a.lastattempt',
                'attemptresult', 'a.attemptresult',
                'state', 'a.state',

            );
        }

        parent::__construct($config);
    }


	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = JFactory::getApplication('administrator');

		// Load the filter state.
		$search = $app->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		$published = $app->getUserStateFromRequest($this->context.'.filter.state', 'filter_published', '', 'string');
		$this->setState('filter.state', $published);
        
        
		//Filtering interval
		$this->setState('filter.interval', $app->getUserStateFromRequest($this->context.'.filter.interval', 'filter_interval', '', 'string'));

		//Filtering backuptype
		$this->setState('filter.backuptype', $app->getUserStateFromRequest($this->context.'.filter.backuptype', 'filter_backuptype', '', 'string'));

		//Filtering attemptresult
		$this->setState('filter.attemptresult', $app->getUserStateFromRequest($this->context.'.filter.attemptresult', 'filter_attemptresult', '', 'string'));

        
		// Load the parameters.
		$params = JComponentHelper::getParams('com_cloudbackup');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.id', 'asc');
	}

	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param	string		$id	A prefix for the store id.
	 * @return	string		A store id.
	 * @since	1.6
	 */
	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id.= ':' . $this->getState('filter.search');
		$id.= ':' . $this->getState('filter.state');

		return parent::getStoreId($id);
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return	JDatabaseQuery
	 * @since	1.6
	 */
	protected function getListQuery()
	{
		// Create a new query object.
		$db		= $this->getDbo();
		$query	= $db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'a.*'
			)
		);
		$query->from('`#__cloudbackup_profiles` AS a');




    // Filter by published state
    $published = $this->getState('filter.state');
    if (is_numeric($published)) {
        $query->where('a.state = '.(int) $published);
    } else if ($published === '') {
        $query->where('(a.state IN (0, 1))');
    }
    

		// Filter by search in title
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			if (stripos($search, 'id:') === 0) {
				$query->where('a.id = '.(int) substr($search, 3));
			} else {
				$search = $db->Quote('%'.$db->escape($search, true).'%');
			}
		}
        


		//Filtering interval
		$filter_interval = $this->state->get("filter.interval");
		if ($filter_interval) {
			$query->where("a.interval = '".$filter_interval."'");
		}

		//Filtering backuptype
		$filter_backuptype = $this->state->get("filter.backuptype");
		if ($filter_backuptype) {
			$query->where("a.backuptype = '".$filter_backuptype."'");
		}

		//Filtering attemptresult
		$filter_attemptresult = $this->state->get("filter.attemptresult");
		if ($filter_attemptresult) {
			if($filter_attemptresult == 'unknown') {
				$query->where("(a.attemptresult <> 'success' AND a.attemptresult <> 'failed')");
			} else {
				$query->where("a.attemptresult = '".$filter_attemptresult."'");
			}
		}			
        
		// Add the list ordering clause.
        $orderCol	= $this->state->get('list.ordering');
        $orderDirn	= $this->state->get('list.direction');
        if ($orderCol && $orderDirn) {
            $query->order($db->escape($orderCol.' '.$orderDirn));
        }

		return $query;
	}
}
