/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 *
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* globals angular,_ */

angular.module('starter.helper', [])
	.factory('ItemEditorHelper', function() {
		"use strict";

		var templateRender = null;

		var ItemEditorHelper = {

			POSTTHIS_DEFAULT: 1,
			POSTTHIS_NO: 2,
			POSTTHIS_YES: 3,

			setHtmlAdvancedAttrs: function(message) {
				var form = 'form[name=adminForm]', $form, element;

				element = window.parent.document.getElementById('autotweet_advanced_attrs');

				if (element)
				{
					message = JSON.stringify(message);
					element.value = message;

					return true;
				}

				message = _.escape(JSON.stringify(message));

				element = '<input type="hidden" id="autotweet_advanced_attrs" name="autotweet_advanced_attrs" value="'
						+ message
						+ '">';

				$form = window.parent.jQuery(form);

				// Form found
				if ($form.size() == 1) {
					$form.append(element);
				} else {

					// Zoo form?
					$form = window.parent.jQuery('form[name=submissionForm]');

					if ($form.size() == 1) {
						$form.append(element);
					}
				}

				return true;
			},

			getHtmlAdvancedAttrs: function() {
				var input, message;

				input = window.parent.document.getElementById('autotweet_advanced_attrs');
				if ((input) && (!_.isEmpty(input.value)))
				{
					message = input.value;
					message = JSON.parse(_.unescape(message));

					return message;
				}

				return null;
			},

			getHtmlEditorText: function() {
				var editor, text;

				if ((window.parent.tinymce) && (window.parent.tinymce.activeEditor)) {
					text = window.parent.tinymce.activeEditor.getContent();

					// No text, but in Zoo there's a change to load it via editors
					if (_.isEmpty(text)) {
						text = window.parent.tinymce.editors[0].getContent();
					}

					return text;
				} else if (editor = window.parent.document.getElementById('jform_articletext')) {

					return editor.value;
				}
			},

			setHtmlPanel: function(panel, title) {
				var element, tabs;

				// com_content - J3 - admin
				element = window.parent.jQuery('#item-form #myTabContent');
				if (element.size())
				{
					ItemEditorHelper.addHtmlTab(title);
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_content - J3 - front
				element = window.parent.jQuery('.item-page #adminForm .tab-content');
				if (element.size())
				{
					ItemEditorHelper.addHtmlTab(title);
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_content - J25 - admin
				element = window.parent.jQuery('#item-form .pane-sliders');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_content - J25 - front
				element = window.parent.jQuery('.item-page #adminForm');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_autotweet requests qTypeTabs
				element = window.parent.jQuery('.request-edit #qContent');
				if (element.size())
				{
					ItemEditorHelper.addHtmlTab(title);
					ItemEditorHelper.addHtmlPanel(element, panel);
					tabs = window.parent.jQuery('#filterconditions-tab a');

					if (tabs.length)
					{
						tabs.tab('show');
					}

					return true;
				}

				// com_k2
				element = window.parent.jQuery('#adminFormK2Sidebar');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_zoo - admin
				element = window.parent.jQuery('#parameter-accordion');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_zoo - front
				element = window.parent.jQuery('#item-submission');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_easyblog - admin - J3
				element = window.parent.jQuery('#eblog-wrapper #options');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_easyblog - admin - J25
				element = window.parent.jQuery('#eblog-wrapper #content-pane');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_easyblog - front
				element = window.parent.jQuery('#widget-writepost .ui-modbody');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_flexicontent - J3 - admin
				element = window.parent.jQuery('#fcform_tabset_0');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					element.height('100%');
					return true;
				}

				// com_jreviews
				element = window.parent.jQuery('.jr-tabs');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// com_jcalpro
				element = window.parent.jQuery('#JCalProEventTabContent');
				if (element.size())
				{
					ItemEditorHelper.addHtmlTab(title);
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// General Joomla 3 - Component
				element = window.parent.jQuery('#content');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				// General Joomla 2.5 - Component
				element = window.parent.jQuery('#element-box div.m');
				if (element.size())
				{
					ItemEditorHelper.addHtmlPanel(element, panel);
					return true;
				}

				return false;
			},

			addHtmlPanel: function(sidebar, panel) {
				var elem;

				elem = window.parent.document.getElementById('autotweet-advanced-attrs');

				if (elem) {
					elem.parentNode.removeChild(elem);
				}

				sidebar.append(panel);
			},

			loadPanel: function(advancedAttrs) {
				var panel, tabs;

				if (_.isEmpty(advancedAttrs)) {
					advancedAttrs = window.parent.autotweetAdvancedAttrs;
				}

				if (_.isEmpty(advancedAttrs)) {
					return false;
				}

				if (templateRender) {
					panel = templateRender(advancedAttrs);
					ItemEditorHelper.setHtmlPanel(panel, advancedAttrs.editorTitle);
				}

				jQuery.ajax({
		            url : window.parent.autotweetPanelTemplate,
		            dataType: "text",
		            success : function (template) {
						templateRender = _.template(template),

						panel = templateRender(advancedAttrs);
						ItemEditorHelper.setHtmlPanel(panel, advancedAttrs.editorTitle);
		            }
		        });
				
				// First tab on the caller
				tabs = parent.jQuery('ul.nav-tabs li a:first');

				if (tabs.length) {
					tabs.tab('show');
				}
			},

			addHtmlTab: function(title) {
				var tabs;

				// Joomla 3 - Admin
				tabs = window.parent.jQuery('#myTabTabs');
				if (tabs.size()) {
					ItemEditorHelper._addHtmlTab(title, tabs);
					return true;
				}

				// Joomla 3 - Front
				tabs = window.parent.jQuery('#adminForm ul.nav-tabs');
				if (tabs.size()) {
					ItemEditorHelper._addHtmlTab(title, tabs);
					return true;
				}

				// JCalPro
				tabs = window.parent.jQuery('#JCalProEventTabTabs');
				if (tabs.size()) {
					ItemEditorHelper._addHtmlTab(title, tabs);
					return true;
				}

				// com_autotweet requests qTypeTabs
				tabs = window.parent.jQuery('.request-edit #qTypeTabs ul.nav-tabs');
				if (tabs.size()) {
					ItemEditorHelper._addHtmlTab(title, tabs);
					return true;
				}

				return false;
			},

			_addHtmlTab: function(title, tabs) {
				var img = '<img src="' + window.parent.autotweetUrlRoot + 'media/com_autotweet/images/autotweet-icon.png">',
					tab;

				if (!window.parent.jQuery('#myAutoTweetTab').size()) {
				    tab = window.parent.jQuery('<li id="myAutoTweetTab" class=""><a id="myAutoTweetTabToogle" href="#autotweet-advanced-attrs" data-toggle="tab">' + img + ' ' + title + '</a></li>');
					tabs.append(tab);

					window.parent.jQuery('#myAutoTweetTabToogle').click(function (e) {
						e.preventDefault();

						if (_.isFunction(jQuery(this).tab)) {
							jQuery(this).tab('show');
						}
					});
				}
			},

			retrieveTitle: function() {
				var title;

				// Joomla
				title = window.parent.jQuery('#jform_title').val();
				if (!_.isEmpty(title)) {
					return title;
				}

				return '';
			},

			retrieveImages: function() {
				var text, $dummyNode, imgs, img, i, $i, imgsrc, imgalt;

				text = ItemEditorHelper.getHtmlEditorText();
				text = text.replace(/src/g, 'data_src');
				
				$dummyNode = jQuery(text),
				
				imgs = [];
				
				_.each($dummyNode.find('img'), function(i) {
					$i = jQuery(i);					
					img = {
						src: $i.attr('data_src'),
						alt: $i.attr('alt')
					};
					imgs.push(img);
				})

				// Joomla
				imgsrc = window.parent.jQuery('#jform_images_image_intro').val();
				if (!_.isEmpty(imgsrc)) {
					imgalt = window.parent.jQuery('#jform_images_image_intro_alt').val() || window.parent.jQuery('#jform_images_image_intro_caption').val();
					img = {
						src: imgsrc,
						alt: imgalt
					};
					imgs.push(img);
				}

				imgsrc = window.parent.jQuery('#jform_images_image_fulltext').val();
				if (!_.isEmpty(imgsrc)) {
					imgalt = window.parent.jQuery('#jform_images_image_fulltext_alt').val() || window.parent.jQuery('#jform_images_image_fulltext_caption').val();
					img = {
						src: imgsrc,
						alt: imgalt
					};
					imgs.push(img);
				}

				// K2
				imgsrc = window.parent.jQuery('.k2AdminImage').attr('src');
				if (!_.isEmpty(imgsrc)) {
					imgsrc = imgsrc.slice(0, imgsrc.indexOf('_S.jpg'));

					imgalt = window.parent.jQuery('input[name=image_caption]').val();

					// XS
					img = {
						src: imgsrc + '_XS.jpg',
						alt: imgalt + ' XS'
					};
					imgs.push(img);

					// S
					img = {
						src: imgsrc + '_S.jpg',
						alt: imgalt + ' S'
					};
					imgs.push(img);

					// Generic
					img = {
						src: imgsrc + '_Generic.jpg',
						alt: imgalt + ' Generic'
					};
					imgs.push(img);

					// M
					img = {
						src: imgsrc + '_M.jpg',
						alt: imgalt + ' M'
					};
					imgs.push(img);

					// L
					img = {
						src: imgsrc + '_L.jpg',
						alt: imgalt
					};
					imgs.push(img);

					// XL
					img = {
						src: imgsrc + '_XL.jpg',
						alt: imgalt + ' XL'
					};
					imgs.push(img);
				}

				// Zoo
				imgsrc = window.parent.jQuery('.image-preview img');
				if (imgsrc.size()) {
					_.each(imgsrc, function(source) {
						img = {
								src: source.src
						};

						if (source.src.toLowerCase().match(/\.(jpg|jpeg|png|gif)$/)) {
							imgs.push(img);
						}
					});
				}

				// EasyBlog
				imgsrc = window.parent.jQuery('.imageData').val();
				if (!_.isEmpty(imgsrc)) {
					imgsrc = JSON.parse(_.unescape(imgsrc));
					img = {
						src: imgsrc.url,
						alt: imgsrc.title
					};
					imgs.push(img);
				}

				// E-Shop
				imgsrc = window.parent.jQuery('input[name=product_image]+img').attr('src');
				if (!_.isEmpty(imgsrc)) {
					imgs.push({
						src: imgsrc
					});
				}

				imgsrc = window.parent.jQuery('#product_images_area img');
				if (imgsrc.size()) {
					_.each(imgsrc, function(source) {
						img = {
								src: source.src
						};
						imgs.push(img);
					});
				}

				return imgs;
			},

			setYesNo: function(v, $field) {
				var data_value;

				if ((v) && (v != $field.val())) {
					data_value = $field.parent().find('a[data-value=' + v + ']');

					// If there's a button
					if (data_value.size())	{
						data_value.click();
					} else {
						$field.val(v);
						$field.trigger('liszt:updated');
					}
				}
			}
		};

		ItemEditorHelper.loadPanel();

		if (window.parent.autotweetAdvancedAttrs) {

			// Loading values from the plugin itself
			ItemEditorHelper.setHtmlAdvancedAttrs(window.parent.autotweetAdvancedAttrs);
		}

		return ItemEditorHelper;
	});
/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 *
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

angular.module('starter.jquery-extras', [])
	.run(function() {
		angular.element(document).ready(function() {
			'use strict';
			
			// Ease on ChosenJS
			jQuery('#itemEditorTabsContent .xt-tab-pane').addClass("tab-pane fade");

			// Tabs
			var tabs = jQuery('#itemEditorTabs a:first');

			if (tabs.length) {
				tabs.tab('show');
			}

			// First tab on the caller
			tabs = window.parent.jQuery('ul.nav-tabs li a:first');

			if (tabs.length) {
				tabs.tab('show');
			}
			
		});
	});

/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 *
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 *
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/**
 * Services that persists and retrieves Agendas from localStorage
 */
angular.module('starter.agenda-service', [])
.factory('Agenda', function() {
		'use strict';

		var STORAGE_ID = 'agendas-autotweet',
			_this = this;
		
		_this.get = function () {
			return JSON.parse(localStorage.getItem(STORAGE_ID) || '[]');
		};
		
		_this.put = function (todos) {
			localStorage.setItem(STORAGE_ID, JSON.stringify(todos));
		};
		
		_this.clear = function () {				
			this.put([]);
		};
		
		return {
			get: _this.get,
			put: _this.put,	
			clear: _this.clear
		};

});
/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* globals angular, _, CryptoJS */

angular
        .module('starter.request-helper', [])
        .factory(
                'RequestHelper',
                [function() {
                  'use strict';

                  var _this = this;

                  _this.formatDatesTimes = function(agendas) {
                    var dates = [];

                    _.each(agendas, function(agenda) {
                      dates.push(agenda.agendaDate + ' ' + agenda.agendaTime);
                    });

                    return dates;
                  };

                  _this.hasPastDates = function(agendas) {
                    var today = new Date(), isPastDate = false;

                    if ((!agendas) || (agendas.length === 0)) { return false; }

                    try {
                      _
                              .each(
                                      agendas,
                                      function(dayTime) {
                                        var d, dateString = dayTime;

                                        if (dayTime
                                                .match(/^\d\d\d\d-\d\d-\d\d \d?\d:\d\d$/)) {
                                          dateString = dayTime
                                                  .replace(' ', 'T')
                                                  + ':00';
                                        }

                                        if (dayTime
                                                .match(/^\d\d\d\d-\d\d-\d\d \d?\d:\d\d:\d\d$/)) {
                                          dateString = dayTime
                                                  .replace(' ', 'T');
                                        }

                                        d = new Date(dateString);

                                        if (d.getTime() < today.getTime()) {
                                          isPastDate = true;
                                        }
                                      });
                    } catch (e) {
                      isPastDate = true;
                    }

                    return isPastDate;
                  };

                  _this.getChannels = function(channels) {
                     if ( (_.isNumber(channels)) && (channels > 0) ) {
                       return [channels];
                     }

                     if ( (_.isString(channels)) && (parseInt(channels) > 0) ) {
                       return [channels];
                     }

                     if (_.isArray(channels)) {
                       return channels;
                     }

                     return [];
                  };

                  _this.getChannelsText = function(channels) {
                    var options, channels_text;

                    if (!window.channelchooser) { return ''; }

                    options = angular.element(window.channelchooser).find(
                            'option');

                    options = _.filter(options, function(option) {
                      return (option.selected) && (parseInt(option.value) > 0);
                    });

                    channels_text = _.reduce(options, function(memo, option) {
                      var e = angular.element(option), txt = e.text();

                      if (_.isEmpty(memo)) {
                        return txt;
                      } else {
                        return memo + ', ' + txt;
                      }
                    }, '');

                    return channels_text;
                  };

                  _this.isValidAgendaRepeat = function(agendas, unix_mhdmd) {
                    var invalid = ((unix_mhdmd) && (unix_mhdmd.length > 0) && (agendas.length > 1));

                    return !invalid;
                  };

                  _this.generateRequestHash = function() {
                    var now = new Date(), hash;

                    hash = CryptoJS.MD5('' + now.getTime()
                            + _.random(0, 9007199254740992));
                    hash = CryptoJS.MD5(hash + _.random(0, 9007199254740992));
                    hash = CryptoJS.MD5(hash + _.random(0, 9007199254740992));

                    return hash.toString(CryptoJS.enc.Hex);
                  };
                  
                  _this.isValidCronjobExpr = function(expr) {
                    var testExpr = /^(((([0-9]+)(\,[0-9]+)*)|\*) ){4}((([0-9]+)(\,[0-9]+)*)|\*)$/;

                    return ( (_.isEmpty(expr)) || (testExpr.test(expr)) );
                  };

                  return {
                    formatDatesTimes: _this.formatDatesTimes,
                    hasPastDates: _this.hasPastDates,
                    getChannelsText: _this.getChannelsText,
                    isValidAgendaRepeat: _this.isValidAgendaRepeat,
                    generateRequestHash: _this.generateRequestHash,
                    getChannels: _this.getChannels,
                    isValidCronjobExpr: _this.isValidCronjobExpr
                  };
                }]);
/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* globals angular, _ */

angular.module('starter.message-controller', []).controller(
        'MessageController',
        function($scope, $rootScope) {
          "$scope:nomunge, $rootScope:nomunge";
          'use strict';

          var _this = this || {};
          var local = $scope.messageCtrl;

          local.remainingCount = 0;
          local.remainingCountClass = '';

          local.countRemaining = function() {
            var style, c, h;

            c = (local.description_value ? local.description_value.length : 0);
            h = (_.isEmpty(_this.hashtags_value) ? 0 : _this.hashtags_value.length + 1);

            c = c + h;
            style = 'label';

            if ((c > 0) && (c <= 60)) {
              style = 'label label-success';
            } else if ((c > 60) && (c <= 100)) {
              style = 'label label-warning';
            } else if (c > 100) {
              style = 'label label-important';
            }

            local.remainingCount = c;
            local.remainingCountClass = style;
          };

          _this.updateMessageView = function(e, desc, hash) {
            local.description_value = desc;

            if (window.hashtags) {
              local.hashtags_value = hash;
            }

            local.countRemaining();
          };

          $rootScope.$on('updateMessageView', _this.updateMessageView);

        });/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* globals angular,_ */

angular
        .module('starter.agenda-controller', ['starter.agenda-service'])
        .controller(
                'AgendaController',
                function($scope, $rootScope, Agenda) {
                  "$scope:nomunge, $rootScope:nomunge, Agenda:nomunge";
                  'use strict';

                  var agendaCtrl = $scope.agendaCtrl, agendas = agendaCtrl.agendas = Agenda
                          .get();

                  Agenda.clear();

                  agendaCtrl.add = function() {
                    var scheduling_date = agendaCtrl.scheduling_date_value;
                    var scheduling_time = agendaCtrl.scheduling_time_value;

                    if ((!scheduling_date) || (!scheduling_time)) {
                      return;
                     }

                    scheduling_date = scheduling_date.trim();
                    if (!scheduling_date.length) {
                      return;
                    }

                    agendas.push({
                      agendaDate: scheduling_date,
                      agendaTime: scheduling_time
                    });

                    Agenda.put(agendas);

                    agendaCtrl.scheduling_date_value = '';
                  };

                  agendaCtrl.remove = function(agenda) {
                    agendas.splice(agendas.indexOf(agenda), 1);
                    Agenda.put(agendas);
                  };

                  $rootScope.$on('newRequest', function() {
                    Agenda.clear();
                    agendas = agendaCtrl.agendas = Agenda.get();
                  });

                  $rootScope.$on('loadDate', function(event, schedulingDate) {
                    agendaCtrl.scheduling_date_value = schedulingDate;
                  });

                  $rootScope.$on('loadTime', function(event, schedulingTime) {
                    agendaCtrl.scheduling_time_value = schedulingTime;
                  });

                  $rootScope.$on('loadAgenda', function(event, param) {
                    var output = [];

                    _.each(param, function(item) {
                      var scheduling_date, scheduling_time, parts = item
                              .split(' ');

                      scheduling_date = parts[0];
                      scheduling_time = parts[1];

                      // 14:45:00
                      parts = scheduling_time.split(':');

                      if (parts.length == 3) {
                        parts.splice(2, 1);
                        scheduling_time = parts.join(':');
                      }

                      output.push({
                        agendaDate: scheduling_date,
                        agendaTime: scheduling_time
                      });
                    });

                    Agenda.put(output);
                    agendas = agendaCtrl.agendas = Agenda.get();
                    agendaCtrl.scheduling_date_value = '';
                  });
                });
angular.module('starter.imagepicker-controller', ['starter.helper'])
.controller('ImagePickerController', function($scope, $rootScope, ItemEditorHelper) {
  "$scope:nomunge,$rootScope:nomunge,ItemEditorHelper:nomunge";
	'use strict';

	var local = $scope.imagePickerCtlr,
	    _this = this;

	_this.loadImages = function() {
		var rawimages = ItemEditorHelper.retrieveImages(), images = [];
		
		images.push({
			value: '',
			text: '-Select-',
			data_img_src: ''
		});

		_.each(rawimages, function(img) {
			var opt = {}, source;
			
			source = img.src;
			source = source.replace(parent.autotweetUrlBase, '');

			opt.value = source;
			opt.text = (_.isEmpty(img.alt) ? source : img.alt);

			if (source.indexOf('http') == -1) {
				source = parent.autotweetUrlRoot + source;
			};
			
			opt.data_img_src = source;
			images.push(opt);
		});
		
		local.images = images;
		$scope.$digest();

		// Delay to jQuery imagepicker
		setTimeout(function() {
			jQuery('#imagechooser').imagepicker({show_label: true})
				.css('display', 'block')
				.css('margin-bottom', '9px');

			jQuery('.image_picker_selector li').first().remove();			
		}, 1); 
	};
	
	local.selectedImage = function() {
		$rootScope.$emit('selectedImage', local.imagechooser_value);
	}
	
	$rootScope.$on('loadImages', _this.loadImages);	

});
/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 *
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* globals angular,_ */

angular.module('starter.itemeditor-controller', [
	'starter.helper',
	'starter.agenda-service',
	'starter.request-helper',
	'ui.bootstrap.buttons',
	'localytics.directives'])
	.controller('ItemEditorController', function($scope, $rootScope, ItemEditorHelper, RequestHelper, Agenda) {
	  "$scope:nomunge,$rootScope:nomunge,ItemEditorHelper:nomunge,RequestHelper:nomunge,Agenda:nomunge";
		'use strict';

		var local = $scope.itemEditorCtrl,
			_this = this;
		
		local.showDialog = false;
		local.messageText = '';		
		
		_this.assignImage = function(e, image_url) {
			if (!_.isEmpty(image_url)) {
				local.image_url_value = image_url;
			}
		};		
		
		_this.itemeditorInit = function() {
			var attrs;
			
			attrs = ItemEditorHelper.getHtmlAdvancedAttrs();
			if (attrs) {				
				$rootScope.$emit('loadMessage', attrs);
			}				
		};
		
		_this.loadMessage = function(e, message) {
			// Shared with MessageController
			$rootScope.$emit('updateMessageView', message.description, message.hashtags);

			local.fulltext_value = message.fulltext;
			
			$rootScope.$emit('loadImages');
			local.image_url_value = message.image_url;

			// Shared with AgendaController			
			$rootScope.$emit('loadAgenda', message.agenda);
			
			// Radio Buttons - Generate click to set values - delay to avoid inprog error
			setTimeout(function() {
				angular.element(document.getElementById('ctrl_itemeditor_postthis_' 
						+ message.postthis)).click();
				angular.element(document.getElementById('ctrl_itemeditor_evergreen_' 
						+ message.evergreen)).click();				
			}, 1);			
			
			local.channelchooser_value = message.channels;
			
			// Shared with CronjobExprController field
			angular.element(unix_mhdmd).val(message.unix_mhdmd);			
			
			local.repeat_until_value = message.repeat_until;
		};
		
		local.onSubmit = function () {
			var descr, message, agendas, unix_mhdmd_value;
			
			// Read it from Message Controller field - description_value
			descr = angular.element(description).val();
			descr = descr.trim();
			
			message = parent.autotweetAdvancedAttrs || {};
			message.description = descr;
			
			// Read it from Message Controller field - hashtags_value
			message.hashtags = angular.element(hashtags).val();
			
			message.fulltext = local.fulltext_value;
			
			// Radio Buttons
			message.postthis = angular.element(itemeditor_postthis).val();
			message.evergreen = angular.element(itemeditor_evergreen).val();
			
			agendas = Agenda.get();
			agendas = RequestHelper.formatDatesTimes(agendas);
			
			// Check Agenda
			if (RequestHelper.hasPastDates(agendas))
			{
				_this.error({
					statusText : 'Agenda has dates in the past. Please, remove them.'
				});

				return false;
			}			
			
			// From CronjobExprController field - unix_mhdmd_value
			unix_mhdmd_value = angular.element(unix_mhdmd).val();
			
      // Valid Expression
      if (!RequestHelper.isValidCronjobExpr(unix_mhdmd_value)) {
        _this.error({
          statusText: 'Repeat Expression is invalid.'
        });

        return false;
      }			
			
			// Check Agenda
			if (!RequestHelper.isValidAgendaRepeat(agendas, unix_mhdmd_value)) {
				_this.error({
					statusText : 'Repeat Expression not allowed for more than one Agenda date.'
				});

				return false;
			}
			
			message.agenda = agendas;
			message.unix_mhdmd = unix_mhdmd_value;			
			message.repeat_until = local.repeat_until_value;

			// Just in jQuery Selection
			if (_.isEmpty(local.image_url_value)) {
				local.image_url_value = angular.element(image_url).val(); 
			}			
			
			message.image_url = local.image_url_value;
			message.channels = local.channelchooser_value;
			message.channels_text = RequestHelper.getChannelsText(local.channelchooser_value);

			ItemEditorHelper.loadPanel(message);
			ItemEditorHelper.setHtmlAdvancedAttrs(message);

			window.parent.xtModalClose();
		};
		
		_this.error = function(msg) {
			local.showDialog = true;
			local.messageText = msg.statusText;
		};		
		
		$rootScope.$on('loadMessage', _this.loadMessage);
		$rootScope.$on('selectedImage', _this.assignImage);
		$rootScope.$on('itemeditorInit', _this.itemeditorInit);
	})
	.run(function($rootScope) {
		angular.element(document).ready(function() {
			$rootScope.$emit('itemeditorInit');
		});
	});
/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 *
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

angular.module('starter', [
	'starter.agenda-controller',	
	'starter.message-controller',	
	'starter.cronjob-expr-controller',
	'starter.imagepicker-controller',
	'starter.jquery-extras',
	'starter.itemeditor-controller'	
	])

	.config(['$compileProvider', function ($compileProvider) {
	  "$compileProvider:nomunge";
		$compileProvider.debugInfoEnabled(false);
	}]);
