<?php

/**
 * @package     Extly.Components
 * @subpackage  PlgContentAutotweetOpenGraph - Plugin AutoTweet NG OpenGraph Tags-Extension for Joomla!
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG New User-Plugin - AutoTweet NG Component is not installed or not enabled.');

	return;
}

include_once JPATH_ADMINISTRATOR . '/components/com_autotweet/helpers/autotweetbase.php';

/**
 * PlgContentAutotweetOpenGraph class.
 *
 * @package     Extly.Plugins
 * @subpackage  autotweetopengraph
 * @since       1.0
 */
class PlgContentAutotweetOpenGraph extends JPlugin
{
	protected $article;

	protected $openGraph;

	protected $classNames;

	protected $ogTagsAdded = false;

	protected static $opt_off = 0;

	protected static $opt_title_article = 1;

	protected static $opt_title_custom = 2;

	protected static $opt_type_article = 1;

	protected static $opt_type_website = 2;

	protected static $opt_type_book = 3;

	protected static $opt_type_profile = 4;

	protected static $opt_type_custom = 5;

	protected static $opt_img_prio_first = 1;

	protected static $opt_img_prio_intro = 2;

	protected static $opt_img_prio_full = 3;

	protected static $opt_img_prio_class = 4;

	protected static $opt_img_prio_custom = 5;

	protected static $opt_locale_site = 1;

	protected static $opt_locale_article = 2;

	protected static $opt_locale_custom = 3;

	protected static $opt_sitename_site = 1;

	protected static $opt_sitename_custom = 2;

	protected static $opt_author_article = 1;

	protected static $opt_author_custom = 2;

	protected static $opt_desc_meta = 1;

	protected static $opt_desc_intro = 2;

	protected static $opt_desc_site = 3;

	protected static $opt_desc_custom = 4;

	/**
	 * PlgContentAutotweetOpenGraph.
	 *
	 * @param   string  &$subject  Params
	 * @param   array   $config    Params
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		JLoader::register('OpengraphHelper', dirname(__FILE__) . '/opengraph.php');
		JLoader::register('OGArticleFactory', JPATH_ADMINISTRATOR . '/components/com_autotweet/helpers/oghelpers/articlefactory.php');
	}

	/**
	 * Method is called by the view
	 *
	 * @param   string   $context     The context of the content passed to the plugin
	 * @param   object   &$article    Content object. Note $article->text is also available
	 * @param   object   &$params     Content params
	 * @param   integer  $limitstart  The 'page' number
	 *
	 * @return	void
	 */
	public function onContentPrepare($context, &$article, &$params, $limitstart=0)
	{
		if ($this->ogTagsAdded)
		{
			return;
		}

		$component = JFactory::getApplication()->input->get('option');

		if ($articleHelper = OGArticleFactory::getHelper($component, $context, $article))
		{
			$this->article = $articleHelper->getArticle();
			$this->prepareOpenGraph();

			$this->openGraph->insertTags();
			$this->ogTagsAdded = true;
		}
	}

	/**
	 * prepareOpenGraph
	 *
	 * @return	void
	 */
	protected function prepareOpenGraph()
	{
		$this->openGraph = new OpengraphHelper;
		$this->openGraph->title = $this->title();
		$this->openGraph->type = $this->type();
		$this->openGraph->description = $this->desc();
		$this->openGraph->locale = $this->locale();
		$this->openGraph->siteName = $this->siteName();
		$this->openGraph->author = $this->author();
		$this->openGraph->imgSrc = $this->image();
		$this->openGraph->url = $this->url();
		$this->openGraph->timePublish = $this->timePublish();
		$this->openGraph->timeModified = $this->timeModified();
		$this->openGraph->section = $this->section();
		$this->openGraph->fbAppId = $this->fbAppID();
	}

	/**
	 * title
	 *
	 * @return	string
	 */
	protected function title()
	{
		$title = '';

		switch ($this->params->def('og-title', 1))
		{
			case self::$opt_title_article :
				$title = $this->article->title;
				break;
			case self::$opt_title_custom :
				$title = $this->params->def('og-title-custom');
				break;
			default:
				break;
		}

		return $title;
	}

	/**
	 * type
	 *
	 * @return	string
	 */
	protected function type()
	{
		$type = '';

		switch ($this->params->def('og-type', 1))
		{
			case self::$opt_type_article:
				$type = 'article';
				break;
			case self::$opt_type_website:
				$type = 'website';
				break;
			case self::$opt_type_book:
				$type = 'book';
				break;
			case self::$opt_type_profile:
				$type = 'profile';
				break;
			case self::$opt_type_custom:
				$type = $this->params->def('og-type-custom');
				break;
			default:
				break;
		}

		return $type;
	}

	/**
	 * desc
	 *
	 * @return	string
	 */
	protected function desc()
	{
		$desc = '';

		switch ($this->params->def('og-desc', 1))
		{
			case self::$opt_desc_meta:
				$desc = $this->article->metadesc;
				break;
			case self::$opt_desc_intro:
				$desc = $this->article->introtext;
				break;
			case self::$opt_desc_site:
				$joomlaConfig = JFactory::getConfig();
				$joomlaSiteName = $joomlaConfig->get('MetaDesc');
				$desc = $joomlaSiteName;
				break;
			case self::$opt_desc_custom:
				$desc = $this->params->def('og-desc-custom');
				break;
			default:
				break;
		}

		$desc = TextUtil::cleanText($desc);
		$desc = TextUtil::truncString($desc, 512);

		return $desc;
	}

	/**
	 * locale
	 *
	 * @return	string
	 */
	private function locale()
	{
		$language = JFactory::getLanguage();
		$locale = str_replace("-", "_", $language->getTag());

		switch ($this->params->def('og-locale', 1))
		{
			case self::$opt_locale_custom:
				$locale = str_replace("-", "_", $this->params->def('og-locale-custom'));
				break;
		}

		return $locale;
	}

	/**
	 * siteName
	 *
	 * @return	string
	 */
	protected function siteName()
	{
		$joomlaConfig = JFactory::getConfig();
		$siteName = $joomlaConfig->get('sitename');

		switch ($this->params->def('og-sitename', 1))
		{
			case self::$opt_sitename_custom:
				$siteName = $this->params->def('og-sitename-custom');
				break;
			default:
				break;
		}

		return $siteName;
	}

	/**
	 * author
	 *
	 * @return	string
	 */
	protected function author()
	{
		$author = $this->getAuthor();

		switch ($this->params->def('og-author', 1))
		{
			case self::$opt_author_custom:
				$author = $this->params->def('og-author-custom');
				break;
			default:
				break;
		}

		return $author;
	}

	/**
	 * getImageWithClass
	 *
	 * @param   string  $images    Param
	 * @param   string  $cssClass  Param
	 *
	 * @return	string
	 */
	protected function getImageWithClass($images, $cssClass)
	{
		if ( (!isset($images)) || (empty ($images))  )
		{
			return;
		}

		foreach ($images as $image)
		{
			$classes = explode(' ', $image['class']);

			if (in_array($cssClass, $classes))
			{
				return $image['src'];
			}
		}
	}

	/**
	 * imageWithOption
	 *
	 * @param   string  $option  Param
	 *
	 * @return	string
	 */
	protected function imageWithOption($option)
	{
		switch ($option)
		{
			case self::$opt_img_prio_first:
				$image = (isset($this->article->firstContentImage)) ? $this->article->firstContentImage : '';
				break;
			case self::$opt_img_prio_intro:
				$image = (isset($this->article->introImage)) ? $this->article->introImage : '';
				break;
			case self::$opt_img_prio_full:
				$image = (isset($this->article->fullTextImage)) ? $this->article->fullTextImage : '';
				break;
			case self::$opt_img_prio_class:
				$images = (!empty($this->article->imageArray)) ? $this->article->imageArray : null;
				$cssClass = $this->params->def('og-img-class', 'xt-image');
				$classImage = $this->getImageWithClass($images, $cssClass);
				$image = (isset($classImage)) ? $classImage : '';
				break;
			case self::$opt_img_prio_custom:
				if ($this->params->def('og-img-custom') != '')
				{
					$image = 'images/' . $this->params->def('og-img-custom');
				}
				break;
			default:
			return;
				break;
		}

		if (!empty($image))
		{
			$image = RouteHelp::getInstance()->getAbsoluteUrl($image, true);

			return $image;
		}
	}

	/**
	 * image
	 *
	 * @return	string
	 */
	protected function image()
	{
		$image = $this->imageWithOption($this->params->def('og-img-prio1', 2));

		if ($image == "" )
		{
			$image = $this->imageWithOption($this->params->def('og-img-prio2', 3));
		}

		if ($image == "" )
		{
			$image = $this->imageWithOption($this->params->def('og-img-prio3', 1));
		}

		return $image;
	}

	/**
	 * url
	 *
	 * @return	string
	 */
	protected function url()
	{
		$url = $this->article->url;

		return $url;
	}

	/**
	 * timePublish
	 *
	 * @return	string
	 */
	protected function timePublish()
	{
		return date('c', strtotime($this->article->publish_up));
	}

	/**
	 * timeModified
	 *
	 * @return	string
	 */
	protected function timeModified()
	{
		return date('c', strtotime($this->article->modified));
	}

	/**
	 * section
	 *
	 * @return	string
	 */
	protected function section()
	{
		return $this->article->category_title;
	}

	/**
	 * author
	 *
	 * @return	string
	 */
	protected function getAuthor()
	{
		$user = JUser::getInstance($this->article->created_by);

		return $user->name;
	}

	/**
	 * fbAppID
	 *
	 * @return	string
	 */
	protected function fbAppID()
	{
		return $this->params->def('og-fbappid');
	}
}
