<?php

/**
 * @package     Extly.Components
 * @subpackage  PlgContentAutotweetOpenGraph - Plugin AutoTweet NG OpenGraph Tags-Extension for Joomla!
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Opengraph class.
 *
 * @package     Extly.Plugins
 * @subpackage  autotweetopengraph
 * @since       1.0
 */
class OpengraphHelper
{
	protected $title;

	protected $type;

	protected $description;

	protected $locale;

	protected $siteName;

	protected $imgSrc;

	protected $url;

	protected $timePublish;

	protected $timeModified;

	protected $section;

	protected $author;

	protected $fbAppId;

	protected static $OG_TITLE = 'og:title';

	protected static $OG_TYPE = 'og:type';

	protected static $OG_DESC = 'og:description';

	protected static $OG_LOCALE = 'og:locale';

	protected static $OG_SITENAME = 'og:site_name';

	protected static $OG_IMAGE = 'og:image';

	protected static $OG_URL = 'og:url';

	protected static $OG_TIMEPUB = 'article:published_time';

	protected static $OG_TIMEMOD = 'article:modified_time';

	protected static $OG_SECTION = 'article:section';

	protected static $OG_AUTHOR = 'article:author';

	protected static $OG_FBAPPID = 'fb:app_id';

	/**
	 * __construct - Class for inserting OG-tags in the HTML header
	 *
	 * @param   string  $title         Param
	 * @param   string  $type          Param
	 * @param   string  $desc          Param
	 * @param   string  $locale        Param
	 * @param   string  $siteName      Param
	 * @param   string  $img           Param
	 * @param   string  $url           Param
	 * @param   string  $timePublish   Param
	 * @param   string  $timeModified  Param
	 * @param   string  $section       Param
	 * @param   string  $author        Param
	 * @param   string  $fbAppId       Param
	 */
	public function __construct($title='', $type='', $desc='', $locale='', $siteName='', $img='', $url='', $timePublish='', $timeModified='', $section='', $author='', $fbAppId='')
	{
		$this->title = $title;
		$this->type = $type;
		$this->description = $desc;
		$this->locale = $locale;
		$this->siteName = $siteName;
		$this->imgSrc = $img;
		$this->url = $url;
		$this->timePublish = $timePublish;
		$this->timeModified = $timeModified;
		$this->section = $section;
		$this->author = $author;
		$this->fbAppId = $fbAppId;
	}

	/**
	 * Method for creating and inserting meta tags in the html header
	 *
	 * @param   string  $property  Param
	 * @param   string  $content   Param
	 *
	 * @return	void
	 */
	protected function insertTag($property, $content)
	{
		$document = JFactory::getDocument();
		$doctype    = $document->getType();

		if ($doctype !== 'html' || $content == '')
		{
			return;
		}

		$sanitizedContent = htmlentities(strip_tags($content), ENT_QUOTES, "UTF-8");
		$meta = '<meta property="' . $property . '" content="' . $sanitizedContent . '" />';
		$document->addCustomTag($meta);
	}

	/**
	 * Method for inserting tags in the HTML header
	 *
	 * @return	void
	 */
	public function insertTags()
	{
		$this->insertTag(self::$OG_TITLE, $this->title);
		$this->insertTag(self::$OG_TYPE, $this->type);
		$this->insertTag(self::$OG_DESC, $this->description);
		$this->insertTag(self::$OG_LOCALE, $this->locale);
		$this->insertTag(self::$OG_SITENAME, $this->siteName);
		$this->insertTag(self::$OG_IMAGE, $this->imgSrc);
		$this->insertTag(self::$OG_URL, $this->url);
		$this->insertTag(self::$OG_TIMEPUB, $this->timePublish);
		$this->insertTag(self::$OG_TIMEMOD, $this->timeModified);
		$this->insertTag(self::$OG_SECTION, $this->section);
		$this->insertTag(self::$OG_AUTHOR, $this->author);
		$this->insertTag(self::$OG_FBAPPID, $this->fbAppId);
	}

	/**
	 * __set
	 *
	 * @param   string  $name   Param
	 * @param   string  $value  Param
	 *
	 * @return	void
	 */
	public function __set($name, $value)
	{
		$this->$name = $value;
	}

	/**
	 * __get
	 *
	 * @param   string  $name  Param
	 *
	 * @return	mixed
	 */
	public function __get($name)
	{
		return $this->$name;
	}
}
