<?php
/**
 * @package     Accusoft.PrizmImage
 * @subpackage  plg_prizmimage
 *
 * @copyright   Copyright (C) 2013 - 2015 Accusoft Corporation, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

 
class PlgContentPrizmImage extends JPlugin
{
    var $version = "1.0.1";

    protected $autoloadLanguage = true;
    protected $componentParams = null;
    protected $componentInstalled = false;
    protected $componentEnabled = false;
    
   	/**
	 * Constructor
	 *
	 * @access      protected
	 * @param       object  $subject The object to observe
	 * @param       array   $config  An array that holds the plugin configuration
	 * @since       1.0
	 */
	public function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);
                
        // Check if Prizm Image Component is installed and enabled
        $is_enabled = PrizmImageHelper::isPrizmImageComponentEnabled( );
        if ( $is_enabled != null )
        {
            $this->componentInstalled = true;
            $this->componentEnabled = $is_enabled;
        }
           
        // Get the parameters from the Prizm Image Component        
        if ( $this->componentInstalled )
            $this->componentParams = JComponentHelper::getParams( 'com_prizmimage' );

        /**
        * Constants
        */
        define( 'IMAGE_SERVICE_TIMEOUT_DEFAULT',  120);
        define( 'IMAGE_SERVICE_TIMEOUT_MIN',  5);
        define( 'IMAGE_SERVICE_TIMEOUT_MAX',  300);
        
		define( 'IMAGE_SERVICE_BASE_URL',     'http://api.prizmimage.com:80/' );
        define( 'IMAGE_SERVICE_POST_SYNC_URL', IMAGE_SERVICE_BASE_URL . '%s/reduce/%s?qualityFactor=%s&removeMetadata=%s&jpegMode=%s' );
        
        // Three quality settings are exposed as user configurable options.
        define('IMAGE_SERVICE_QUALITY_LOW', 28);
        define('IMAGE_SERVICE_QUALITY_MED', 22);
        define('IMAGE_SERVICE_QUALITY_HIGH', 16);
        
        // User Agent used in HTTP messages.
        define( 'IMAGE_SERVICE_UA', "Joomla PrizmImage/{$this->version} (+http://extensions.joomla.org/extensions/core-enhancements/performance/site-performance/)" );
		
		define( 'IMAGE_SERVICE_DATABASE', '#__prizmimage' );
        
        if ( $this->componentParams )
        {
            define( 'IMAGE_SERVICE_USER_ID', $this->componentParams->get('pi_userid') );
            define( 'IMAGE_SERVICE_JPEG_MODE', $this->componentParams->get('pi_jpegMode') );
            define( 'IMAGE_SERVICE_QUALITY_FACTOR', $this->componentParams->get('pi_qualityFactor') );
            define( 'IMAGE_SERVICE_DEBUG', $this->componentParams->get('pi_debug') ); 
            define( 'IMAGE_SERVICE_AUTO', $this->componentParams->get('pi_processOnUpload') == 'Yes' );
        }
        else
        {
            define( 'IMAGE_SERVICE_USER_ID', '' );
            define( 'IMAGE_SERVICE_JPEG_MODE', 'progressive' );
            define( 'IMAGE_SERVICE_QUALITY_FACTOR', strval(IMAGE_SERVICE_QUALITY_MED) );
            define( 'IMAGE_SERVICE_DEBUG', false ); 
            define( 'IMAGE_SERVICE_AUTO', false );               
        }

        // Load language file
        JFactory::getLanguage()->load('plg_content_prizmimage');
        
	}

    public function onContentBeforeSave($context, &$article, $isnew)
    {
        // bail out if not being called from the upload() in 'com_media'
        if ($context != 'com_media.file')
            return true;
            
        // Continue only if the Prizm Image Component is installed
        if (!$this->componentInstalled)
            return true;
            
        // Continue only if automatic processing for uploads is enabled
        if (!IMAGE_SERVICE_AUTO)
            return true;
 
        // Call onPrizmImageReduce directly
        $this->onPrizmImageReduce($context, $article, $isnew, true);
        
	    return true;
    }
    
    // Event that calls the Prizm Image Service
    // Same parameters as an onContentBeforeSave
    // $article is really an $object_file object with the following fields:
    //     tmp_name - the path to the file or temporary file that will be reduced
    //     filepath - the final path of the file that will be reduced
    //     size - the size of the 'orginal' file in bytes
    // $displayMessage - Boolean to determine if final status message is displayed, overriddened by IMAGE_SERVICE_DEBUG
    public function onPrizmImageReduce($context, &$article, $isnew, $displayMessage)
    {
        // bail out if not being called from the upload() in 'com_media' or from reduce() in 'com_prizmimage'
        if (!($context == 'com_media.file' || $context == 'com_prizmimage.file'))
            return true;

        // Get a handle to the Joomla! application object
        $app = JFactory::getApplication();       
        $object_file = $article;
        
        // Display debugging enabled message when debug is on
        if (IMAGE_SERVICE_DEBUG)
        {
            $app->enqueueMessage(JText::_('PLG_CONTENT_PRIZMIMAGE_DEBUG_ENABLED'), 'Warning');
            // Override $displayMessage when debug is on
            $displayMessage = true;
        }
                    
        // Check to make sure this is a valid image type handled by the image service
        if (!PrizmImageHelper::file_is_handled_by_image_service($object_file->filepath))
        {
            if ( IMAGE_SERVICE_DEBUG )
                $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_INVALID_FILETYPE', JFile::getName($object_file->filepath)), 'Warning');
            return true;
        }
  
        $http = JHttpFactory::getHttp();
        // Use the tmp_name because that is the path from all calls to this event
        $tempfile_path = $object_file->tmp_name;
        $file_path = $object_file->filepath;        
        $headers = array('user-agent' => IMAGE_SERVICE_UA);
        $file_contents = file_get_contents($tempfile_path);       
		
        // Get the remove metadata from the plugin configuration parameters
        if ($this->componentParams->get('pi_removeMetadata'))
            $remove_metadata="true";
        else
            $remove_metadata="false";
                
        // Get the timeout threshold from the plugin configuration parameters
        $timeout = strval($this->componentParams->get('pi_timeout'));
        if ($timeout == 0)
            $timeout = IMAGE_SERVICE_TIMEOUT_DEFAULT;
        else if ($timeout < IMAGE_SERVICE_TIMEOUT_MIN)
            $timeout = IMAGE_SERVICE_TIMEOUT_MIN;
        else if ($timeout > IMAGE_SERVICE_TIMEOUT_MAX)
            $timeout = IMAGE_SERVICE_TIMEOUT_MAX;
        
        // Note the timeout value in debug mode
        if ( IMAGE_SERVICE_DEBUG )
            $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_TIMEOUT_VALUE', $timeout), 'Message');                             
        
        // Our API requires that the user id be prepended to the request and the file name be appended to the request, followed by options.
        $req = JText::sprintf(IMAGE_SERVICE_POST_SYNC_URL, IMAGE_SERVICE_USER_ID, rawurlencode(basename($file_path)), 
                   IMAGE_SERVICE_QUALITY_FACTOR, $remove_metadata, IMAGE_SERVICE_JPEG_MODE);

        // Invoke the POST request.
        $response = $http->post($req, $file_contents, $headers, $timeout);

        // Always note the returned status code in debug mode
        if ( IMAGE_SERVICE_DEBUG )
            $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_SERVER_STATUS_DEBUG', $response->code), 'Message');

        // default to success with no file difference
        $success = true;
        $status = 0;
        $message = '';
        $newFilesize = $object_file->size;
        
        if ($response->code == 200)
        {
            file_put_contents($tempfile_path, $response->body);
            $newFilesize = strlen($response->body);
        }
        else
        {
            $success = false;
            $status = -1;

            // The body of the message will contain JSON content with additional error information. Since the data returned from
            // the post has been written to a file, get the contents of the file.  Then extract the JSON content error message from the body.
            $responseData = $this->json_decode_message($response->body);
            if ($responseData)
            {
                $message = $responseData->message;
            }
            else
            {
                $message = JText::sprintf('PLG_CONTENT_PRIZMIMAGE_SERVER_STATUS', $response->code);
            }           
        }

        // Get the file name only just for display
        $fileNameOnly = JFile::getName($object_file->filepath);
        
        // update the Prizm Image database
		$filepath = str_replace(DIRECTORY_SEPARATOR, '/', $object_file->filepath);
        $result = $this->updateDatabase(PrizmImageHelper::getRelativeMediaPath($filepath), $object_file->size, $newFilesize, $status);
        if (IMAGE_SERVICE_DEBUG)
        {
            if ($result == true)
                $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_DATABASE_ADD_SUCCESS', $fileNameOnly), 'Message');
            else
                $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_DATABASE_ADD_FAIL', $fileNameOnly), 'Warning');
        }

        // Display success or fail message
        if ($success)
        {
            if ($newFilesize != $object_file->size)
            {
                // Add to the total bytes saved
                $totalBytesSaved = $this->componentParams->get('pi_totalBytesSaved');
                $totalBytesSaved += ($object_file->size - $newFilesize);
                PrizmImageHelper::setPrizmImageParam('pi_totalBytesSaved', $totalBytesSaved);
                
                if ($displayMessage)
                    $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_REDUCE_SUCCESS', $fileNameOnly, $object_file->size, $newFilesize), 'Message');
            }
            else
                if ($displayMessage)
                    $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_REDUCE_NOSAVINGS', $fileNameOnly), 'Message');                
        }
        else
        {
            $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_REDUCE_FAIL', $fileNameOnly, $message ), 'Error');
        }
            
	    return true;
    }
    
    public function onContentAfterDelete($context, $article)
	{       
        // bail out if not being called from the delete() in 'com_media'
        if ($context != 'com_media.file')
            return true;

        // Get a handle to the Joomla! application object
        $app = JFactory::getApplication();       
        $object_file = $article;
                    
        // Display debugging enabled message when debug is on
        if (IMAGE_SERVICE_DEBUG)
        {
            $app->enqueueMessage(JText::_('PLG_CONTENT_PRIZMIMAGE_DEBUG_ENABLED'), 'Warning');
        }
            
        // Check to make sure this is a valid image type handled by the image service
        if (!PrizmImageHelper::file_is_handled_by_image_service($object_file->filepath))
        {
            return true;
        }
        
        // Check to see if the file is in the PrizmImage database
        $fileId = $this->getFileDatabaseId( PrizmImageHelper::getRelativeMediaPath($object_file->filepath) );       
        if ($fileId != 0)
        {
            $result = $this->deleteFileFromDatabase($fileId);
            if ( IMAGE_SERVICE_DEBUG )
            {
                if ( $result == true )
                    $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_DATABASE_DELETE_SUCCESS', JFile::getName($object_file->filepath)), 'Message');
                else
                    $app->enqueueMessage(JText::sprintf('PLG_CONTENT_PRIZMIMAGE_DATABASE_DELETE_FAIL', JFile::getName($object_file->filepath)), 'Warning');
            }

        }
        
        return true;
	}
    
    // Checks to see if a file exists in the PrizmImage database
    // Returns the id of the database record if the file is in the PrizmImage database
    // Otherwise it returns 0 if the file does not exist in the PrizmImage database
    function getFileDatabaseId( $filepath )
    {
        $fileId = 0;
        
        // Get the database
   		$db = JFactory::getDBO();
                
        // check to see if the file is in the database
        $query = $db->getQuery(true);
        $query->select($db->quoteName('id'));
        $query->from($db->quoteName(IMAGE_SERVICE_DATABASE));
		$query->where($db->quoteName('filePath') . ' = ' . $db->quote($filepath));
        $db->setQuery((string)$query);
        $db->query();
        $fileRow = $db->loadRow( );
        
        if ($fileRow)
        {
            // record exists
            // update new reduced size and last reduced date
            $fileId = $fileRow['0'];
        }
        
        return $fileId;
    }
    
    // Update the PrizmImage database by either
    // inserting a new file record or updating an existing one
    function updateDatabase( $filepath, $originalFilesize, $newFilesize, $status )
    {
        // Get the database
   		$db = JFactory::getDbo();
        
        $fileId = $this->getFileDatabaseId( $filepath );
        
        if ($fileId != 0)
        {
            // record exists, now update record
            $query = $db->getQuery(true);            
            // Fields to update.
            $date = JFactory::getDate()->toSql(false, $db);
            $fields = array(
                $db->quoteName('reducedFileSize') . ' = ' . $db->quote($newFilesize),
                $db->quoteName('lastReduceDate') . ' = ' . $db->quote($date),
                $db->quoteName('lastReduceStatus') . ' = ' . $db->quote($status)
                );
            $conditions = array( $db->quoteName('id') . ' = ' . $db->quote($fileId) );
 
            $query->update($db->quoteName(IMAGE_SERVICE_DATABASE))->set($fields)->where($conditions); 
            $db->setQuery($query); 
            $result = $db->query();
        }
        else
        {
            // new file to database, insert new record 
            $query = $db->getQuery(true);                        
            $columns = array($db->quoteName('createDate'), $db->quoteName('filePath'), $db->quoteName('originalFileSize'), $db->quoteName('reducedFileSize'), $db->quoteName('lastReduceDate'), $db->quoteName('lastReduceStatus'));
            $date = JFactory::getDate()->toSql(false, $db);
            $values = array($db->quote($date), $db->quote($filepath), $db->quote($originalFilesize), $db->quote($newFilesize), $db->quote($date), $db->quote($status));
            $query
                ->insert($db->quoteName(IMAGE_SERVICE_DATABASE))
                ->columns($columns)
                ->values(implode(',', $values));
            $db->setQuery($query);
            $result = $db->query();            
        }
        
        return $result;
    }
    
    // Deletes a file from the PrizmImage database
    // where the 'id' equals the passed in $fileId
    function deleteFileFromDatabase( $fileId )
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        // delete record where id = $fileId
        $conditions = array( $db->quoteName('id') . ' = ' . $db->quote($fileId) );
        $query
            ->delete($db->quoteName(IMAGE_SERVICE_DATABASE))
            ->where($conditions);
        $db->setQuery($query);
        $result = $db->query();
        return $result;
    }
    
    // This function makes sure json processing code is present and calls it.
    function json_decode_message($data)
    {
        if ( function_exists('json_decode') )
        {
            $data = json_decode( $data );
        }
        else
        {
            require_once( 'JSON/JSON.php' );
            $json = new Services_JSON( );
            $data = $json->decode( $data );
        }
        return $data;
    }
     
} // end plgContentPrizmImage

class PrizmImageHelper
{
    // Determine if the image type is handled by Prizm Image.
    // Currently we handle JPG, PNG, and GIF.
    public static function file_is_handled_by_image_service( $file )
    {
        $ext = strtolower(JFile::getExt($file));
		switch ($ext)
		{
			// Image
			case 'jpg':
            case 'jpeg':
			case 'png':
			case 'gif':
                return true;
            default:
                return false;
        }						
    }
    
    public static function getRelativeMediaPath($filePath, $requireLeadingDirSeparator=true)
    {
        $com_media_base = COM_MEDIA_BASE;
        if (!$requireLeadingDirSeparator)
            $com_media_base .= '/';
        $mediaBase = str_replace(DIRECTORY_SEPARATOR, '/', $com_media_base);
        $relativePath = str_replace($mediaBase, '', $filePath);
        return $relativePath;
    }
    
    public static function setPrizmImageParam($paramName, $value)
    {
        // Get the params and set the new values
        $params = JComponentHelper::getParams('com_prizmimage');
        $params->set($paramName, $value);

        // Get a new database query instance
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);

        // Build the query
        $query->update('#__extensions AS a');
        $query->set('a.params = ' . $db->quote((string)$params));
        $query->where('a.element = "com_prizmimage"');

        // Execute the query
        $db->setQuery($query);
        $db->query();
    }

    // Check if Prizm Image Component is installed and enabled
    // Component is not installed if the return value is null
    // Otherwise, the enabled status of the Prizm Image Component 
    public static function isPrizmImageComponentEnabled()
    {
        // Check if Prizm Image Component is installed and enabled
        $component = 'com_prizmimage';
        $db = JFactory::getDbo();
        $dbTable = '#__extensions';
        $conditions = array( $db->quoteName('name') . ' = ' . $db->quote($component) ); 
        $query = $db->getQuery(true);
        $query->select($db->quoteName('enabled'))
              ->from($db->quoteName($dbTable))
              ->where($conditions);
        $db->setQuery((string)$query);
        $is_enabled = $db->loadResult();
        
        return $is_enabled;        
    }
    
} // end PrizmImageHelper

?>