<?php

/**
 * @package     Extly.Components
 * @subpackage  plgButtonJoocialEditor - Plugin Joocial Editor Button
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Editor JoocialEditor buton
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PlgButtonJoocialEditor extends JPlugin
{
	/**
	 * Constructor
	 *
	 * @param   object  &$subject  The object to observe
	 * @param   array   $config    An array that holds the plugin configuration
	 *
	 * @since       1.5
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		// Com_jreviews,
		$integrated_components = $this->params->get('integrated_components', 'com_autotweet,com_content,com_easyblog,com_flexicontent,com_jcalpro,com_k2,com_zoo');
		$this->integrated_components = explode(',', $integrated_components);

		$this->loadLanguage();

		// Load component language file for use with plugin
		$jlang = JFactory::getLanguage();
		$jlang->load('com_autotweet');

		include_once JPATH_ADMINISTRATOR . '/components/com_autotweet/helpers/autotweetbase.php';
	}

	/**
	 * Display the button
	 *
	 * @param   string  $name  Param
	 *
	 * @return array A four element array of (article_id, article_title, category_id, object)
	 */
	public function onDisplay($name)
	{
		$jinput = JFactory::getApplication()->input;
		$comp = $jinput->get('option');

		if (!in_array($comp, $this->integrated_components))
		{
			return false;
		}

		$link = AutoTweetDefaultView::addItemeditorHelperApp();

		JHtml::_('bootstrap.modal');

		$doc = JFactory::getDocument();
		$icon = JUri::root() . '/media/com_autotweet/images/joocialeditorbutton.png';

		if (EXTLY_J35)
		{
			$css = '.icon-joocial-editor { background: url("' . $icon . '") no-repeat scroll 2px 2px transparent!important;}';
		}
		else
		{
			$css = '.icon-joocial-editor { background: url("' . $icon . '") no-repeat scroll 100% 0 transparent;}';
		}

		$doc->addStyleDeclaration($css);

		Extly::loadAwesome();

		echo JHtmlBootstrap::renderModal(
				'joocial_editor_modal',
				array(
						'url' => $link,
						'title' => JText::_('PLG_JOOCIALEDITOR_BUTTON'),
						'height' => '400px',
						'width' => '500px',
						'class' => 'joocial_editor_modal'
				)
		);

		$doc->addScriptDeclaration("
	// Joocial Editor
	if (!window.xtModalClose) {
		window.xtModalClose = function() {
			jQuery('div.modal').modal('hide');
		};
	};
	");

		$button = new JObject;

		if (EXTLY_J35)
		{
			$button->modal = false;
			$button->class = 'btn';
			$button->onclick = "jQuery('.icon-joocial-editor').parent()
				.attr('href', '#joocial_editor_modal')
				.attr('role', 'button')
				.attr('data-toggle', 'modal');";
			$button->title = JText::_('PLG_JOOCIALEDITOR_BUTTON');
			$button->text = JText::_('PLG_JOOCIALEDITOR_BUTTON');
			$button->name = 'joocial-editor';
			$button->link = '#';
		}
		else
		{
			$button->name = 'joocial-editor';
			$button->class = 'btn joocial_editor_modal';
			$button->modal = false;
			$button->link = false;
			$button->onclick = "jQuery('.joocial_editor_modal').attr('href', '#joocial_editor_modal').attr('role', 'button').attr('data-toggle', 'modal'); ";
			$button->title = JText::_('PLG_JOOCIALEDITOR_BUTTON');
			$button->text = JText::_('PLG_JOOCIALEDITOR_BUTTON');
		}

		return $button;
	}
}
