<?php
/**
 * @version             1.1.3 EXTENDED
 * @package             ampz
 * @copyright           Copyright (C) 2014 roosterz.nl. All rights reserved.
 * @license             GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * Joomla! is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.plugin.plugin');

class plgSystemAmpz extends JPlugin
{
    public function __construct(&$subject, $config)
    {
        // Placement
        $this->_inline = false;
        $this->_sidebar = false;
        $this->_flyin = false;
        $this->_mobile = false;

        // Networks
        $this->_networks = array();
        $this->_noEnabledNetworks = false;

        //Counts
        $this->_inlineShareCounts = true;
        $this->_sidebarShareCounts = true;
        $this->_flyinShareCounts = true;
        $this->_mobileShareCounts = true;
        $this->_InlineShareCountsClass = '';

        // Threshold for total shares display
        $this->_thresholdTotalShares = 0;

        // Entrance Effect for the flyin location
        $this->_flyinEntranceEffect = "";

        // Button Size
        $this->_buttonSize = "";
        $this->_buttonSizeClass = "";

        // Rounded Buttons
        $this->_roundedButtons = false;
        $this->_roundedButtonsClass = "";

        $this->_currentComponent = "";

        $this->_continue = 0;
        $this->_frontpage = 0;
        $this->_title = "";
        $this->_emailSubject = "";
        $this->_url = "";
        $this->_baseUrl = "";
        $this->_buttonFontSize = 0;
        $this->_extra_classes = "ampz_colorbg";
        $this->_buttonTemplate = "template_amsterdam";

        // Mobile
        $this->_mobileNumberOfButtons = 3;
        $this->_mobileButtonWidth = 0;
        $this->_mobileButtonWidthStyle = "";

        // Open in New Tab or Popup (default)
        $this->_openInNewTab = false;

        // Support Author
        $this->_disableSupportAuthor = false;

        parent::__construct($subject, $config);
        $this->loadLanguage();
    }

    /************************
     * onAfterRoute
     *************************/
    public function onAfterRoute()
    {
        $app = JFactory::getApplication();

        $this->_currentComponent = $app->input->get("option");
        $excludeCatView = $this->params->get("exclude_category_view");
        $this->_mobile = $this->params->get("mobileEnable"); // Mobile Mode activated?

        if ($this->continueOrNot($app, $excludeCatView))
            $this->_continue = 1;
    }

    /************************
     * onBeforeCompileHead
     *************************/
    public function onBeforeCompileHead()
    {
        if (!$this->_continue)
            return;
    }

    /************************
     * onBeforeRender: needed for the inline display
     *************************/
    public function onBeforeRender()
    {
        if (!$this->_continue)
            return;

        // Add CSS
        $document = JFactory::getDocument();
        $document->addStyleSheet(JURI::root(true) . '/plugins/system/ampz/ampz/css/ampz.min.css');
        $document->addStyleSheet(JURI::root(true) . '/plugins/system/ampz/ampz/css/animate.min.css');

        $this->_buttonTemplate = $this->params->get("buttonTemplate"); // Get the selected design preset for the buttons
        $this->getExtraCSSClasses($this->_buttonTemplate); // add extra CSS classes if necessary

        // Include Open Sans Font
        $document->addStyleSheet('//fonts.googleapis.com/css?family=Open+Sans:400,600');

        $this->_buttonFontSize = $this->params->get("buttonSize"); // Get selected button size

        $this->_inlineShareCounts = $this->params->get("inline_share_counts"); // Show inline share counts or not
        $this->_inlineShareCounts != "1" ? $this->_inlineShareCountsClass = 'ampz_no_count' : $this->_inlineShareCountsClass = '';
        $this->_flyinShareCounts = $this->params->get("flyin_share_counts"); // Show flyin share counts or not
        $this->_flyinShareCounts != "1" ? $this->_flyinShareCountsClass = 'ampz_no_count' : $this->_flyinShareCountsClass = '';
        $this->_sidebarShareCounts = $this->params->get("sidebar_share_counts"); // Show sidebar share counts or not
        $this->_sidebarShareCounts != "1" ? $this->_sidebarShareCountsClass = 'ampz_no_count' : $this->_sidebarShareCountsClass = '';
        $this->_mobileShareCounts = $this->params->get("mobile_share_counts"); // Show mobile share counts or not
        $this->_mobileShareCounts != "1" ? $this->_mobileShareCountsClass = 'ampz_no_count' : $this->_mobileShareCountsClass = '';
        $this->_thresholdTotalShares = $this->params->get("threshold_total_shares"); // Threshold Total Shares

        $inlineNumColumns = $this->params->get("inlineNumColumns");

        $positionFlyIn = $this->params->get("positionFlyin");
        $positionFlyIn == "left" ? $positionFlyInClass = "ampz_flyin_left " : $positionFlyInClass = 'ampz_flyin_right ';
        $this->_flyinEntranceEffect = $positionFlyInClass . $this->params->get("entranceFlyIn");

        $inlineTopBottom = $this->params->get('inlineTopBottom');

        $placements = array();
        if ($this->_inline) {
            if (strpos($inlineTopBottom, "bottom") !== false)
                $placements[] = 'inline_bottom';
            if (strpos($inlineTopBottom, "top") !== false)
                $placements[] = 'inline_top';
        }
        $this->_sidebar ? $placements[] = 'sidebar' : '';
        $this->_flyin ? $placements[] = 'flyin' : '';
        $this->_mobile ? $placements[] = 'mobile' : '';

        $mobileWidth = $this->params->get("mobile_width"); // Activate Mobile Mode if browser is less than

        $entranceDelay = strval($this->params->get("entrance_delay")) . "s"; // Get the entrance delay

        // Get the enabled networks
        $enabledNetworks = $this->params->get("AMPZ_NETWORKS_SELECT");
        if (!empty($enabledNetworks)) {
            $this->_networks = $enabledNetworks;
            $firstNetwork = substr($this->_networks[0], 4, -2);
        }
        else {
            $this->_noEnabledNetworks = true;
            JFactory::getApplication()->enqueueMessage('AMPZ Notice: you have enabled the ampz plugin, but not enabled any social networks. Please enable one or more networks in Settings -> Networks. More information can be found <a href="https://roosterz.gitbooks.io/ampz/content/networks.html" target="_blank">here</a>.', 'message');
            return;
        }

        // Support author or not
        $this->_disableSupportAuthor = $this->params->get("disable_support_author");

        // Get url to be shared
        $this->_url = $this->getUrl("");
        //$this->_url = "https://www.roosterz.nl/joomla-extensions/ampz";
        $this->_baseUrl = JURI::root();

        // Start Caching
        $counts = array(); // initialize the counts array
        $cache_lifetime = intval($this->params->get("cache_lifetime")) * 60; // cache lifetime in minutes
        $cached_counts = false;

        // Fly In Triggers
        $this->params->get("flyin_trigger_bottom") ? $flyInTriggerBottom = true : $flyInTriggerBottom = false;
        $this->params->get("flyin_trigger_time") ? $flyInTriggerTime = true : $flyInTriggerTime = false;
        $flyInTriggerTimeSeconds = intval($this->params->get("flyin_trigger_time_seconds")) * 1000;

        // Mobile Trigger
        $this->params->get("mobile_trigger_scroll") ? $mobileTriggerScroll = true : $mobileTriggerScroll = false;


        if ($cache_lifetime != 0) // Caching Feature enabled
        {
            require_once 'ampz/js/helpers/cache.class.php';

            $cacheId = "ampz_" . $this->_url;

            $cache = new Cache(array(
                'name'      => $cacheId,
                'path'      => JPATH_BASE . '/cache/plg_ampz/',
                'extension' => '.cache'
            ));

            $cache->eraseExpired(); // Erase all expired entries
            $counts = $cache->retrieveAll(); // Retrieve all the cached counts, if there are any

            !empty($counts) ? $cached_counts = true : $cached_counts = false; // if empty than either cache is expired or not cached at all
        }
        // End Caching

        if ($this->params->get("open_new_tab")) {
          $this->_openInNewTab = true;
        }

        // Build JS needed vars and add them into document
        $js = "var ampzSettings = ";
        $js_vars = array(
            "ampzCounts" => ($cached_counts ? $counts['ampz'] : ''),
            "ampzPlacement" => $placements,
            "ampzEntranceDelay" => "$entranceDelay",
            "ampzMobileWidth" => "$mobileWidth",
            "ampzFlyinEntranceEffect" => "$this->_flyinEntranceEffect",
            "ampzThresholdTotalShares" => "$this->_thresholdTotalShares",
            "ampzBaseUrl" => "$this->_baseUrl",
            "ampzShareUrl" => "$this->_url",
            "ampzOpenInNewTab" => "$this->_openInNewTab",
            "ampzCacheLifetime" => "$cache_lifetime",
            "ampzCachedCounts" => "$cached_counts",
            "ampzFlyInTriggerBottom" => "$flyInTriggerBottom",
            "ampzFlyInTriggerTime" => "$flyInTriggerTime",
            "ampzFlyInTriggerTimeSeconds" => "$flyInTriggerTimeSeconds",
            "ampzInlineNumColumns" => "$inlineNumColumns",
            "ampzMobileTriggerScroll" => "$mobileTriggerScroll"
        );

        $js_vars_json = $js . json_encode($js_vars) . ";";
        $document->addScriptDeclaration($js_vars_json);

        // Add JQuery if Needed
        //$preventJQueryLoad = $this->params->get("prevent_jquery_load");
        if ((substr(JVERSION,0,1) == "2"))// && ($preventJQueryLoad != 1)) // Joomla 2.5
        {
            $document->addScript("https://ajax.googleapis.com/ajax/libs/jquery/1.11.2/jquery.min.js");
        }

        // Add other required javascripts
        $document->addScript(JURI::root(true) . '/plugins/system/ampz/ampz/js/ampz.js');
        $document->addScript(JURI::root(true) . '/plugins/system/ampz/ampz/js/ampz_buttonz.min.js');

        // Generic variables
        $this->_buttonSize = $this->params->get("buttonSize"); // Get the selected button size
        $this->_buttonSize != "btn_normal" ? $this->_buttonSizeClass = $this->_buttonSize : $this->_buttonSizeClass = '';

        $this->_roundedButtons = $this->params->get("roundedButtons"); // Rounded buttons or not?
        $this->_roundedButtons == "1" ? $this->_roundedButtonsClass = 'ampz_rounded' : $this->_roundedButtonsClass = '';

        // Get the Title
        $this->_title = $this->getTitle();
        // Email subject if using that button
        $this->_emailSubject = $this->params->get("subject_email");

        // Start Inline
        if ($this->_inline)
        {
            $doc = JFactory::getDocument();
            $buffer = $doc->getBuffer('component');

            $entranceInline = $this->params->get("entranceInline"); // Get the selected entrance animation
            $entranceInline != "none" ? $entranceInlineClass = "animated " . $entranceInline : $entranceInlineClass = '';

            $hoverEffectInline = $this->params->get("hoverEffectInline"); // Get the selected hover effect
            $hoverEffectInline != "none" ? $hoverEffectInlineClass = $hoverEffectInline : $hoverEffectInlineClass = '';

            $totalSharesInline = $this->params->get("inline_total_shares"); // Display total shares or not
            $totalSharesInline ? $totalSharesInlineDiv = $this->getTotalSharesDiv($this->params->get("Inline_color_total_shares"), $this->params->get("label_total_shares"), $this->_buttonSizeClass, '') : $totalSharesInlineDiv = '';

            $inlineDisplayLabels = $this->params->get('inline_display_labels');
            $inlineDisplayLabels ? $inlineDisplayLabels = 1 : $inlineDisplayLabels = 0;

            $totalSharesInline = $this->params->get("inline_total_shares"); // Display total shares or not
            $totalSharesInline ? $totalSharesInlineDiv = $this->getTotalSharesDiv($this->params->get("inline_color_total_shares"), $this->params->get("label_total_shares"), $this->_buttonSizeClass, '') : $totalSharesInlineDiv = '';

            $inlineDisplayCount = $this->params->get('inline_share_counts');
            $inlineDisplayCount ? $inlineDisplayCount = 1 : $inlineDisplayCount = 0;

            $inlineTopAboveTitle = $this->params->get('inlineTopAboveTitle');

            if (strpos($inlineTopBottom, "top") !== false)
            {
                $InlineTopHtml = $this->generateAmpzDiv("inline_top", $inlineDisplayLabels, $inlineDisplayCount, $totalSharesInlineDiv, $entranceInlineClass, $hoverEffectInlineClass, $this->_InlineShareCountsClass, $this->_disableSupportAuthor);

                $pos = stripos($buffer, '</h'); // find the position of the first closing heading tag (title)

                if ($inlineTopAboveTitle OR $pos === false) { // Defined to show above title or no heading tag found
                    $buffer_new = $InlineTopHtml . $buffer;
                }
                else if ($pos !== false) // title tag is found so insert ampz after title
                {
                    $pos = $pos + 5;
                    $buffer_new = substr_replace($buffer, $InlineTopHtml, $pos, 0);
                }
            }
            if (strpos($inlineTopBottom, "bottom") !== false)
            {
                $InlineBottomHtml = $this->generateAmpzDiv("inline_bottom", $inlineDisplayLabels, $inlineDisplayCount, $totalSharesInlineDiv, $entranceInlineClass, $hoverEffectInlineClass, $this->_InlineShareCountsClass, $this->_disableSupportAuthor);

                isset($buffer_new) ? $buffer = $buffer_new : '';
                $posK2Tag = stripos($buffer, 'K2AfterDisplayContent'); // Is there a default K2AfterDisplayContent tag?

                if ($posK2Tag !== false) { // Yes, so place it right after this tag
                    $posK2Tag = $posK2Tag + 25;

                    $buffer_new = substr_replace($buffer, $InlineBottomHtml, $posK2Tag, 0);
                }
                else { // No K2 tag so just place it after the buffer
                    $buffer_new = $buffer . $InlineBottomHtml;
                }
            }

            // reset the buffer
            $doc->setBuffer($buffer_new, 'component');
        }
    }

    /************************
     * onAfterRender: needed for the sidebar display
     *************************/
    public function onAfterRender()
    {
        if (!$this->_continue || $this->_noEnabledNetworks)
            return;

        // Start Sidebar
        if ($this->_sidebar)
        {
            $html = JResponse::getBody();

            if ($html == '')
            {
                return;
            }

            $positionSidebar = $this->params->get("positionSideBar");
            $positionSidebar == "right" ? $positionSideBarClass = "ampz_sidebar_right " : $positionSideBarClass = 'ampz_sidebar_left ';

            $entranceSideBar = $this->params->get("entranceSideBar"); // Get the selected entrance animation
            $entranceSideBar != "none" ? $entranceSideBarClass = $positionSideBarClass . "animated " . $entranceSideBar : $entranceSideBarClass = $positionSideBarClass;

            $hoverEffectSideBar = $this->params->get("hoverEffectSideBar"); // Get the selected hover effect
            $hoverEffectSideBar != "none" ? $hoverEffectSideBarClass = $hoverEffectSideBar : $hoverEffectSideBarClass = '';

            $totalSharesSideBar = $this->params->get("sidebar_total_shares"); // Display total shares or not
            $totalSharesSideBar ? $totalSharesSideBarDiv = $this->getTotalSharesDiv($this->params->get("sidebar_color_total_shares"), $this->params->get("label_total_shares"), $this->_buttonSizeClass, '') : $totalSharesSideBarDiv = '';

            $sidebarDisplayCount = $this->params->get('sidebar_share_counts');
            $sidebarDisplayCount ? $sidebarDisplayCount = 1 : $sidebarDisplayCount = 0;

            $sidebarHtml = $this->generateAmpzDiv("sidebar", 0, $sidebarDisplayCount, $totalSharesSideBarDiv, $entranceSideBarClass, $hoverEffectSideBarClass, $this->_sidebarShareCountsClass, $this->_disableSupportAuthor) . "</body>";

            $html = str_replace('</body>', $sidebarHtml, $html);

            JResponse::setBody($html);
        }

        // Start Fly In
        if ($this->_flyin)
        {
            $html = JResponse::getBody();

            if ($html == '')
            {
                return;
            }

            $hoverEffectFlyIn = $this->params->get("hoverEffectFlyIn"); // Get the selected hover effect
            $hoverEffectFlyIn != "none" ? $hoverEffectFlyInClass = $hoverEffectFlyIn : $hoverEffectFlyInClass = '';

            $flyInDisplayCount = $this->params->get('flyin_share_counts');
            $flyInDisplayCount ? $flyInDisplayCount = 1 : $flyInDisplayCount = 0;

            $flyInHtml = $this->generateAmpzDiv("flyin", 1, $flyInDisplayCount, '', '', $hoverEffectFlyInClass, $this->_flyinShareCountsClass, $this->_disableSupportAuthor) . "</body>";

            $html = str_replace('</body>', $flyInHtml, $html);

            JResponse::setBody($html);
        }

        // Start Mobile
        if ($this->_mobile)
        {
            $html = JResponse::getBody();

            if ($html == '')
            {
                return;
            }

            $entranceMobile = $this->params->get("entranceMobile"); // Get the selected entrance animation
            $entranceMobile != "none" ? $entranceMobileClass = "animated " . $entranceMobile : $entranceMobileClass = '';

            $totalSharesMobile = $this->params->get("mobile_total_shares"); // Display total shares or not
            $this->_mobileNumberOfButtons = $this->params->get("mobile_number_of_buttons"); // Get the number of buttons on mobile
            $totalSharesMobile ? $this->_mobileButtonWidth = 100/($this->_mobileNumberOfButtons + 1) : $this->_mobileButtonWidth = 100/($this->_mobileNumberOfButtons);
            $this->_mobileButtonWidthStyle = "width: " . $this->_mobileButtonWidth . "%;"; // Inline Style for Mobile button width
            $totalSharesMobile ? $totalSharesMobileDiv = $this->getTotalSharesDiv($this->params->get("mobile_color_total_shares"), $this->params->get("label_total_shares"), $this->_buttonSizeClass, $this->_mobileButtonWidthStyle) : $totalSharesMobileDiv = '';

            $mobileDisplayCount = $this->params->get('mobile_share_counts');
            $mobileDisplayCount ? $mobileDisplayCount = 1 : $mobileDisplayCount = 0;

            $mobileHtml = $this->generateAmpzDiv("mobile", 0, $mobileDisplayCount, $totalSharesMobileDiv, $entranceMobileClass, '', $this->_mobileShareCountsClass, $this->_disableSupportAuthor) . "</body>";

            $html = str_replace('</body>', $mobileHtml, $html);

            JResponse::setBody($html);
        }


    }

    /***************************************************************************************
     * Start Helper Functions
     ****************************************************************************************/

    /************************
     * continueOrNot
     *************************/
    private function continueOrNot($app, $excludeCatView)
    {
        // Admin view or Print view:
        if($app->isAdmin() || ($app->input->post->get('print') == 1)) {
            return false;
        }

        $placement_array = $this->params->get("AMPZ_LOCATION_SELECT"); // Get the enabled locations

        // Start Frontpage Display
        $menu = $app->getMenu();
        $lang = JFactory::getLanguage();

        if ($menu->getActive() == $menu->getDefault($lang->getTag()))
        {
            $this->_frontpage = 1;
            $showOnFrontpageSidebar = $this->params->get("sidebar_frontpage_display");
            $showOnFrontpageFlyin = $this->params->get("flyin_frontpage_display");
            $showOnFrontpageInline = $this->params->get("inline_frontpage_display");

            if (!$showOnFrontpageSidebar && !$showOnFrontpageFlyin && !$showOnFrontpageInline)
                return false;
            else
            {
                if ($showOnFrontpageSidebar  && in_array("display_sidebar", $placement_array))
                    $this->_sidebar = true;
                if ($showOnFrontpageFlyin && in_array("display_flyin", $placement_array))
                    $this->_flyin = true;
                if ($showOnFrontpageInline && in_array("display_inline", $placement_array))
                    $this->_inline = true;
            }

            return true;
        }
        // End Frontpage Display

        $jinput = $app->input;

        // Not on frontpage. Let's check if the current component is enabled
        $includeComponentsArray = $this->params->get("includeComponents");
        if (is_array($includeComponentsArray) && (in_array($this->_currentComponent, $includeComponentsArray))) // Current Component is Included by the User
        {
            if ($this->isCategoryView($this->_currentComponent) && $excludeCatView) {
                return false;
            }

            // Start Check Joomla Content
            if ($this->_currentComponent == "com_content") { // Joomla Native Content Component
                $excludeCats = $this->params->get('excludeCategories');
                $excludeCategories = empty($excludeCats) ? array() : $excludeCats;

                $excludeArts = $this->params->get('excludeArticles');
                $excludeArticles = empty($excludeArts) ? array() : $excludeArts;

                if ($this->isCategoryView($this->_currentComponent)) { // Category View
                    $currentCatId = JRequest::getInt('id');
                    $currentArticleId = 0;
                }
                else { // Article View
                    $currentArticleId = JRequest::getInt('id');
                    // Retrieve Category ID
                    $db = JFactory::getDBO();
                    $db->setQuery("SELECT catid FROM #__content WHERE id = ".$currentArticleId );
                    $currentCatId = $db->loadResult();
                }

                if ( (in_array(strval($currentCatId), $excludeCategories)) || (in_array(strval($currentArticleId), $excludeArticles)) ) // Current article is in a category which is excluded or the article itself is excluded
                    return false;
            }
            // End Check Joomla Content

            // Start Check K2 Content
            if ($this->_currentComponent == "com_k2") { // K2 Component
                $excludeCats = $this->params->get('excludeK2Categories');
                $excludeCategories = empty($excludeCats) ? array() : $excludeCats;

                $excludeArts = $this->params->get('excludeK2Articles');
                $excludeArticles = empty($excludeArts) ? array() : $excludeArts;

                if ($this->isCategoryView($this->_currentComponent)) { // Category View
                    $currentCatId = JRequest::getInt('id');
                    $currentArticleId = 0;
                }
                else { // Article View
                    $currentArticleId = JRequest::getInt('id');
                    $db = JFactory::getDBO();
                    $db->setQuery("SELECT catid FROM #__k2_items WHERE id = ".$currentArticleId );
                    $currentCatId = $db->loadResult();
                }

                if ( (in_array(strval($currentCatId), $excludeCategories)) || (in_array(strval($currentArticleId), $excludeArticles)) ) // Current article is in a category which is excluded or the article itself is excluded
                    return false;
            }
            // End Check K2 Content

            if (!empty($placement_array)) {
                if (in_array("display_inline", $placement_array))
                    $this->_inline = true;
                if (in_array("display_sidebar", $placement_array))
                    $this->_sidebar = true;
                if (in_array("display_flyin", $placement_array))
                    $this->_flyin = true;

                return true;
            }
            else if (empty($placement_array) AND $this->_mobile) { // only mobile position enabled
                return true;
            }
            else { // no position at all enabled
                return false;
            }
        }

        else
            return false;
    }

    /************************
     * getUrl
     *************************/
    private function getUrl()
    {
        $url = 'http';

        if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443)
        {

            $url .= "s";
        }

        $url .= "://" . $_SERVER["SERVER_NAME"] . $_SERVER["REQUEST_URI"];

        return $url;
    }

    /************************
     * getTotalSharesDiv: Get the DIV html for the total shares
     *************************/
    private function getTotalSharesDiv($color, $label, $size, $widthStyle)
    {
        $html = '<div class="ampz_total_shares '. $size .'" style="color: '. $color .'; '. $widthStyle .'"><span class="ampz_total_shares_count">0</span><div class="ampz_total_shares_label">'. $label .'</div></div>';

        return $html;
    }

    /************************
     * getTitle: Get the title for the page to share
     *************************/
    private function getTitle()
    {
        $doc = JFactory::getDocument();
        $title = $doc->getTitle();

        //$this->_title = urlencode($title);

        return $title;
    }

    /************************
     * getExtraCSSClasses: function to get the CSS classes for the chosen button design
     *************************/
    private function getExtraCSSClasses($template)
    {
        switch($template)
        {
            case "template_rotterdam":
            case "template_breda":
            case "template_utrecht":
                $this->_extra_classes = "";
                break;
            case "template_eindhoven":
            $this->_extra_classes = "ampz_colorbg ampz_overlay";
                break;
            case "template_amsterdam":
            $this->_extra_classes = "ampz_colorbg";
                break;
        }
    }

    /************************
     * isCategoryView: returns true if current view is category view
     *************************/
    private function isCategoryView($currentComponent)
    {
        $currentView = JRequest::getWord("view");
        $currentLayout = JRequest::getWord("layout");

        if ($currentView == "category" || $currentView == "featured" || $currentView == "itemlist" || $currentView == "latest" || $currentView == "album" || $currentLayout == "list") {
            return true;
        }
        else if ($currentComponent == "com_zoo") { // special things for ZOO
            $requestUrl = $_SERVER["REQUEST_URI"];
            $regexTrailingSlashNumber = "/\\/\\d+/";

            if ( (preg_match($regexTrailingSlashNumber, $requestUrl) === 1) || ($currentView == "frontpage") ) // ZOO Category pages for some reason
                return true;
        }
        else {
          return false;
        }
    }

    /************************
     * generateAmpzDiv
     *************************/
    private function generateAmpzDiv($position, $displayLabels, $displayCount, $totalSharesDiv, $entranceEffect, $hoverEffect, $shareCountsClass, $disableSupportAuthor)
    {
        $html = '<!-- start ampz '. $position .' -->';
        $html .= '<div id="ampz_'. $position .'" class="'. $entranceEffect .'">';

        if ($position == "flyin")
        {
            $flyinTitle = $this->params->get("headingFlyIn");
            $flyinText = $this->params->get("textFlyIn");

            $html .= '
            <header class="ampz_flyin_header">
                <h3>'. $flyinTitle .'</h3>
                <a href="#" class="ampz_flyin_close">Close</a>
                <p>'. $flyinText .'</p>
	        </header>';
        }

        $html .= '  <div class="'. $this->_roundedButtonsClass .' ampz_container">' . $totalSharesDiv .
                        '<ul>';

        // Generate network html
        $buttonIndex = 1; // used for checking how many buttons have been created in mobile mode
        foreach ($this->_networks as $network)
        {
            $network = substr($network, 4, -2); // only take the network name, not the preceding btn_ and ending _#

            if (($network == "whatsapp" || $network == "telegram") && $position != "mobile") { // If mobile button and not a mobile position, skip it
                continue;
            }

            if ($displayLabels)
            {
                // Retrieve Network Label
                $network_label_text = "label_" . $network;
                $network_label = $this->params->get($network_label_text);
            }

            // Get Twitter via if filled out
            ($network == "twitter" && ($this->params->get("via_twitter") != "")) ? $data_via = 'data-via=' . $this->params->get("via_twitter") : $data_via = '';

            switch ($network) {
                case "pinterest":
                    $href = 'javascript:void((function()%7Bvar%20e=document.createElement(&apos;script&apos;);e.setAttribute(&apos;type&apos;,&apos;text/javascript&apos;);e.setAttribute(&apos;charset&apos;,&apos;UTF-8&apos;);e.setAttribute(&apos;src&apos;,&apos;https://assets.pinterest.com/js/pinmarklet.js?r=&apos;+Math.random()*99999999);document.body.appendChild(e)%7D)());';
                    break;
                case "email":
                    $href_body = $this->_emailSubject . urldecode($this->_title) . '&amp;body=' . $this->_url;
                    $href = "mailto:?subject=$href_body";
                    break;

                default:
                    $href = "#";
            }

            $html .= '
                    <li class="'. $hoverEffect .'" style="'. $this->_mobileButtonWidthStyle .'">
					    <a class="'. $this->_buttonTemplate .' ampz_btn '. $this->_buttonSizeClass .' ampz_'. $network . ' ' . $shareCountsClass . ' ' . $this->_extra_classes .'" data-url="'. $this->_url .'" data-basecount="0" data-shareposition="'. $position .'"  data-sharetype="'. $network . '" data-text="'. $this->_title .'" '. $data_via .' href="'. $href .'">
					        <i class="ampz ampz-icon ampz-icon-'. $network .'"></i>
                  ';

            $displayLabels ? $html .= '<span class="ampz_network_label">'. $network_label . '</span>' : '';
            $displayCount ? $html .= '<span class="ampz_count"></span>' : '';
            ((strpos($this->_extra_classes,'ampz_overlay') !== false) && ($position != "sidebar" && $position != "mobile")) ? $html .= '<span class="ampz_icon_overlay"></span>' : '';

            $html .=        '</a>
                ';

            if (($buttonIndex == $this->_mobileNumberOfButtons) && ($position === "mobile")) {
                break;
            }

            $buttonIndex++;
        }

        $html .= '
                    </ul>';

        if (!$disableSupportAuthor && $position == "sidebar")
        {
          $html .= '
            <div class="ampz_support_author">
					    <a href="http://www.roosterz.nl/joomla-extensions/ampz" class="ampz_roosterz_icon" target="_blank">
              </a>
            </div>';
        }

        $html .= '
                </div>
            </div>';

        $html .= '<!-- end ampz '. $position .' -->';

        return $html;
    }

}

?>
