/**
 * @version             1.1.3 EXTENDED
 * @package             ampz
 * @element             ampz
 * @copyright           Copyright (C) 2015 roosterz.nl. All rights reserved.
 * @license             GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * Joomla! is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */

jQuery(document).ready(function($) {

    // Add delay to attract more attention
    var entranceDelay = ampzSettings.ampzEntranceDelay;
    $('.animated').css({
        'animation-delay': entranceDelay,
        '-webkit-animation-delay': entranceDelay
    });

    var cachingHelper = 'plugins/system/ampz/ampz/js/helpers/caching.php'; // path to cache helper
    var cachedComplete = false; // variable to indicate whether caching has completed
    var thresholdTotalShares = ampzSettings.ampzThresholdTotalShares;
    var cacheLifetime = ampzSettings.ampzCacheLifetime;
    var baseUrl = ampzSettings.ampzBaseUrl;
    var fetchCounts = !ampzSettings.ampzCachedCounts ? true : false; // Fetch counts or use cached counts
    var firstPosition = '#ampz_' + ampzSettings.ampzPlacement[0]; // only first position is needed for the share counts

    var mobileWidth = ampzSettings.ampzMobileWidth;

    if ( ($(window).width() < mobileWidth) && ($.inArray("mobile", ampzSettings.ampzPlacement) != -1)) { // Mobile enabled and window smaller than threshold
        if (ampzSettings.ampzMobileTriggerScroll)
        {
            $(window).scroll(function () {
                $('#ampz_mobile').css('display', 'block');
            });
        }
        else {
            $('#ampz_mobile').css('display', 'block');
        }
    }
    else { // No mobile mode active so display the others
        $('#ampz_inline_top').css('display', 'block');
        $('#ampz_inline_bottom').css('display', 'block');
        $('#ampz_sidebar').css('display', 'block');

        if ( ($.inArray( "inline_top", ampzSettings.ampzPlacement ) != -1) || ($.inArray( "inline_bottom", ampzSettings.ampzPlacement ) != -1) ) // Inline Settings
        {
            var inlineNumColumns = parseInt(ampzSettings.ampzInlineNumColumns);

            if (inlineNumColumns != -1) { // -1 is original width so don't do anything
                var numOfButtons = Math.max($('#ampz_inline_top li').length, $('#ampz_inline_bottom li').length);
                var widthInlineDiv = $('div[id^="ampz_inline_"]').outerWidth();
                var marginButtons = parseInt($('div[id^="ampz_inline_"] li').css('margin-right')) + parseInt($('div[id^="ampz_inline_"] li').css('margin-left'));
                var widthTotalSharesDiv = $('div[id^="ampz_inline_"] .ampz_total_shares').outerWidth();
                var inlineButtonWidth = ( (widthInlineDiv - widthTotalSharesDiv - 5 - numOfButtons * marginButtons) / numOfButtons ) + 'px';

                $('div[id^="ampz_inline_"] li').css('width', inlineButtonWidth); // set the width for the inline buttons
            }
        }
        if ($.inArray( "flyin", ampzSettings.ampzPlacement ) != -1) // Fly In Settings
        {
            if (ampzSettings.ampzFlyInTriggerTime)
            {
                setTimeout(function() {   //calls click event after a certain time
                    showFlyIn();
                }, ampzSettings.ampzFlyInTriggerTimeSeconds);
            }

            if (ampzSettings.ampzFlyInTriggerBottom)
            {
                $(window).scroll(function () {
                    if ($(window).scrollTop() >= $(document).height() - $(window).height() - 10) {
                        showFlyIn();
                    }
                });
            }

            $('#ampz_flyin').on('click', function(event){
                if( $(event.target).is('#ampz_flyin') || $(event.target).is('.ampz_flyin_close') ) {
                    $("#ampz_flyin").hide(600);
                    event.preventDefault();
                }
            });
        }
    }

    // Start fetching and/or setting the counts:
    $(firstPosition).ampz({ counts: ampzSettings.ampzCounts, fetchCounts: fetchCounts, baseUrl: baseUrl, thresholdTotalShares: thresholdTotalShares, openInNewTab: ampzSettings.ampzOpenInNewTab,
        onInit: function () {
            $.each( ampzSettings.ampzPlacement, function( index, value ){
                $('#ampz_' + value).ampz({ attachEvents: true, fetchCounts: false, openInNewTab: ampzSettings.ampzOpenInNewTab,
                }); // attach the button click events to all positions
            });
        },
        onLoad: function() {
            if (fetchCounts && cacheLifetime !== "0") {
                var shareCountArray = ($(firstPosition).ampz('getShareCountArray'));
                if (!cachedComplete) {
                    //console.log(shareCountArray);
                    cacheCount('ampz', shareCountArray, ampzSettings.ampzShareUrl, cacheLifetime);
                }
            }
        }
    });

    function showFlyIn() {
        $('#ampz_flyin').css('display', 'block');
        $('#ampz_flyin').addClass(ampzSettings.ampzFlyinEntranceEffect);
        $('#ampz_flyin').addClass('animated_direct is-visible');
    }

    function cacheCount(name, counts, url, cacheLifetime) {

        var cacheId = 'ampz_' + url;
        //console.log(counts);

        $.ajax({
            url: baseUrl + cachingHelper,
            type: 'post',
            data: { "name": name, "count": counts, "cacheId": cacheId, "expiration": cacheLifetime }
        }).done(function (response) {
            cachedComplete = true;
        }).fail(function (jqXHR,status,err) {
            console.log( "error caching ampz: " + err );
        });
    }

});
