<?php
/**
 * @version     2.1.0
 * @package     com_cloudbackup
 * @subpackage  plg_system_cloudbackup
 * @copyright   Copyright (C) 2012 - 2013. Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://bhartiy.com
 *
 * CloudBackup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * CloudBackup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CloudBackup; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
**/

	// no direct access
	defined('_JEXEC') or die;
	
	/**
	* Joomla! System CloudBackup Plugin
	*
	* @package		Joomla.Plugin
	* @subpackage	System.CloudBackup
	*/
	class  plgSystemCloudbackup extends JPlugin
	{
		private $excludefiles  = array();
		private $excludetables = array();
		private $mailbody 	   = array();
		private $chunksize	   = 40;
		
		public function onAfterInitialise()
		{
			$this->loadLanguage();
			$app = JFactory::getApplication();
			
			if ($app->isAdmin()) return true;
			if(!$this->checkSchedule()) return true;
			
			$started = $this->markAction();
			if($started)
			{
				ini_set('memory_limit', '1024M');
				set_time_limit(0);
				$uploadables = array();
				
				if($this->profile->backuptype == 'full' || $this->profile->backuptype == 'files')
				{
					$this->doFilesBackup($uploadables);
				}
				if($this->profile->backuptype == 'full' || $this->profile->backuptype == 'db')
				{
					$this->doDatabaseBackup($uploadables);
				}
				
				$this->saveFileInfo($uploadables);
				$this->upload2Cloud($uploadables);
				$this->markAction(true);
			}

			$this->notifyAdmin();
		}
		
		private function doDatabaseBackup(&$uploadables)
		{
			$app = JFactory::getApplication();
			$db  = JFactory::getDbo();
			$this->excludetables = array();
			$excludetables = explode("\n", $this->profile->excludetables);
			
			foreach($excludetables as $ii => $vv) {
				$this->excludetables[$ii] = $db->getPrefix() . trim($vv);
			}

			$siteurl   = str_replace('/', '_', JUri::getInstance()->toString(array('host', 'port')). JUri::root(true));
			$timestamp = JHtml::date('now', 'YmdHis');
			
			$sqlfile   = $this->createSqlDump('site-db-'.$timestamp.'-'.$siteurl.'.sql');
			if($sqlfile == false) {
				$this->mailbody[] = 'Error: Failed to create SQL Dump for database. Skipping site database backup.';
			} else {
				$zipdb = $this->createZip($sqlfile, 'site-db-'.$timestamp.'-'.$siteurl.'.sql.zip');
				if($zipdb == false) {
					$uploadables[] 		 = array('filename' => $sqlfile, 'mime' => 'text/plain', 'comment' => 'Database Backup SQL of '. $app->getCfg('sitename'));
					$this->mailbody[]    = 'Error: Failed to create Compressed archive of SQL Dump for database. SQL file will be uploaded.';
				} else {
					unlink($sqlfile);	 // No more needed
					$uploadables[] 		 = array('filename' => $zipdb, 'mime' => 'application/zip', 'comment' => 'Database Backup of '. $app->getCfg('sitename')); 
					$this->mailbody[]    = 'Message: Compressed archive created from SQL Dump for database. Ready for upload.';
				}
			}
		}
		
		private function doFilesBackup(&$uploadables)
		{
			$app = JFactory::getApplication();
			
			$this->excludefiles  = array();
			$excludefiles = explode("\n", $this->profile->excludefiles);
			foreach($excludefiles as $ii => $vv) {
				$this->excludefiles[$ii] = str_replace('\\', '/', realpath(JPATH_SITE.'/'.trim($vv)));
			}
			
			$this->excludefiles[] = str_replace('\\', '/', realpath($app->getCfg('tmp_path')));
			$this->excludefiles[] = str_replace('\\', '/', realpath($app->getCfg('log_path')));
			
			$siteurl   = str_replace('/', '_', JUri::getInstance()->toString(array('host', 'port')). JUri::root(true));
			$timestamp = JHtml::date('now', 'YmdHis');
			
			$zipfile   = $this->createZip(JPATH_SITE, 'site-files-'.$timestamp.'-'.$siteurl.'.zip', $this->excludefiles);
			if($zipfile == false) {
				$this->mailbody[]    = 'Error: Failed to create Compressed archive of website files. Skipping site files.';
			} else {
				$uploadables[] 		 = array('filename' => $zipfile, 'mime' => 'application/zip', 'comment' => 'Files Backup of '. $app->getCfg('sitename')); 
				$this->mailbody[]    = 'Message: Compressed archive created from Website files. Ready for upload.';
			}
		}
		
		private function createSqlDump($filename) 
		{
			$db  = JFactory::getDbo();
			$app = JFactory::getApplication();
			$contents = array();
			$dumpfile = $app->getCfg('tmp_path').'/'.$filename;
			
			$contents[] = '-- Developed by: Bhartiy Web Technologies, Bhartiy.com';
			$contents[] = '-- Backup created on '. JFactory::getDate();
			
			$db->setQuery('SHOW TABLES LIKE \''. $db->getPrefix() . '%\'');
			$tables = $db->loadColumn(0);
			if(empty($tables)) {
				$this->mailbody[] = 'Error: Failed to load database table list.';
				return false;
			}
			
			$create = $db->getTableCreate($tables);
			if(empty($create)) {
				$this->mailbody[] = 'Error: Failed to get table structure for the database tables.';
				return false;
			}
			
			$this->writeBuffer($dumpfile, $contents, true);		// Force overwrite
			
			$tables = array_diff($tables, $this->excludetables);			
			
			$this->mailbody[] = 'Message: Following database tables are included in the backup: '."\n\t". implode("\n\t", $tables);

			foreach($tables as $table)
			{
				$fields 	= array_keys($db->getTableColumns($table));
				foreach($fields as $fid => $fld) 
				{
					$fields[$fid] = $db->qn($fld);
				}
				
				$contents[] = '';
				$contents[] = '';
				$contents[] = '-- TABLE: '.$db->qn($table);
				$contents[] = '';
				$contents[] = 'DROP TABLE IF EXISTS '.$db->qn($table).';';
				$contents[] = $create[$table].';';
				$contents[] = '';
				
				$this->writeBuffer($dumpfile, $contents);
				
				$query      = 'SELECT * FROM '.$db->qn($table);
				$db->setQuery($query);
				$db->query();
				$numRows    = $db->getNumRows();
				$offset     = 0;
				
				while($offset < $numRows)
				{
					$rowSql = array();
					$db->setQuery($query, $offset, $this->chunksize);
					$rows  = $db->loadObjectList();
					
					if($db->getErrorNum()) 
					{
						$this->mailbody[] = 'Error: Failed to load table data for: '. $table;
						break;		// No more data query for this table
					}
					
					foreach($rows as $row)
					{
						foreach($row as $rkey => $rvalue) 
						{
							$row->$rkey = ($rvalue === null) ? 'NULL' : $db->quote($rvalue);
						}
						$rowSql[] = '( ' . implode( ', ', (array)$row ) . ' )';
					}
					
					if(count($rowSql)) 
					{
						$contents[] = 'INSERT INTO '.$db->qn($table). '('.implode(', ', $fields). ') VALUES '."\n". implode(",\n", $rowSql).';';
					}
					
					$this->writeBuffer($dumpfile, $contents);
					$offset += $this->chunksize;
				}
				
				$contents[] = '';
				$contents[] = '-- Backup Completed with '.count($tables).' tables.';
				$contents[] = '-- Job finished on '. JFactory::getDate();
				$this->writeBuffer($dumpfile, $contents);
			}
			
			return $dumpfile;
		}
		
		private function createZip($source, $destination, $exclude = array()) 
		{
			$app = JFactory::getApplication();
			$destination = $app->getCfg('tmp_path').'/'.$destination;
			
			if (!file_exists($source)) return false;
			$zip = new ZipArchive();
			if ( $res = $zip->open( $destination, ZIPARCHIVE::CREATE ) != true ) {
				$this->mailbody[] = 'Error: Failed to create zip file at location : '. $destination;
				return false;
			}
			
			$source = str_replace( '\\', '/', realpath( $source ) );
			if ( is_dir( $source ) === true ) {
				$files = $this->listDir( $source, $exclude, true );
				
				$incfiles = array();
				foreach($files as $cfile)
				{
					$incfiles[] = substr($cfile, strlen(JPATH_SITE));
				}
				$this->mailbody[] = 'Message: Following files are included in the backup: '."\n\t". implode("\n\t", $incfiles) ;
				
				foreach ( $files as $file ) {
					$file = str_replace( '\\', '/', realpath( $file ) );
					if ( is_dir( $file ) === true ) {
						$zip->addEmptyDir( str_replace( $source . '/', '', $file . '/' ) );
					} else if ( is_file( $file ) === true ) {
						$zip->addFromString( str_replace( $source . '/', '', $file ), file_get_contents( $file ));
					}
				}
			} else if (is_file($source) === true) {
				$zip->addFromString(basename($source), file_get_contents($source));
			}
			
			$closed = $zip->close();
			return $closed ? $destination : false;
		}
		
		private function listDir($base_path, $exclude = array(), $recursive = true) 
		{
			$base_path = rtrim($base_path, '/') . '/';
			if (!is_dir($base_path)) return false;
			$exclude_files = array( '.', '..', '.DS_Store', '.svn', 'backup' );
			$result_list = array();
			if ($folder_handle = opendir($base_path)) {
				while ( false !== ( $filename = readdir( $folder_handle ) ) ) {
					if ( !in_array( $filename, $exclude_files ) && !in_array( $base_path . $filename, $exclude ) ) {
						if ( is_dir( $base_path . $filename . '/' ) ) {
							$result_list[] = $base_path . $filename;
							if( $recursive ) {
								$temp_list = $this->listDir( $base_path . $filename . '/', $exclude, $recursive);
								foreach ( $temp_list as $item ) {
									$result_list[] = $item;
								}
							}
						} else {
							$result_list[] = $base_path . $filename;
						}
					}
				}
				closedir($folder_handle);
				return $result_list;
			}
		}
		
		private function writeBuffer($dumpfile, &$contents, $new = false)
		{
			$fh = fopen($dumpfile, ($new ? 'w' : 'a'));
			if($fh) 
			{
				fwrite($fh, implode("\n", $contents) . "\n");
				fclose($fh);
			}
			$contents = array();
		}
		
		private function saveFileInfo(&$uploadables)
		{
			$db 	= JFactory::getDbo();
			$query  = $db->getQuery(true);
			$date   = JFactory::getDate()->toSql();
			
			foreach($uploadables as $i => $uploadable)
			{
				$query->clear();
				$query->insert('#__cloudbackup_files')
					->set('filename  = '. $db->quote(substr($uploadable['filename'], strlen(JPATH_SITE))))
					->set('filesize  = '. $db->quote(filesize($uploadable['filename'])))
					->set('mime  	 = '. $db->quote($uploadable['mime']))
					->set('dateadded = '. $db->quote($date));
				$db->setQuery($query);
				$db->query();
				
				if($db->getErrorNum()) {
					$this->mailbody[]      = 'Error: Failed to save uploadable file info to database. Error was # '.$db->getErrorMsg();
					$uploadables[$i]['id'] = null;
				} else {
					$uploadables[$i]['id'] = $db->insertId();
				}
			}
		}
		
		private function upload2Cloud(&$uploadables)
		{
			$db = JFactory::getDbo();
			JPluginHelper::importPlugin('cloudbackup');
			$dispatcher    = JDispatcher::getInstance();
			$cloudresposes = $dispatcher->trigger('onCloudUpload', array($uploadables, $this->profile->backupfolder, &$this->mailbody));
			
			$uploads = JArrayHelper::pivot($uploadables, 'id');
			foreach($cloudresposes as $cloudrespose)
			{
				foreach($cloudrespose as $backupinfo)
				{
					$backupinfo = (object) $backupinfo;
					if($backupinfo->cloudid)
					{
						$delFile = $uploads[$backupinfo->fileid]['filename'];
						if(file_exists($delFile))
						{
							unlink($delFile);
							$this->mailbody[] = 'Message: '.$uploads[$backupinfo->fileid]['comment'].' was uploaded to atleast one cloud server. Deleted from local directory.';
						}
					}
					$db->insertObject('#__cloudbackup_backups', $backupinfo, 'id');
					// Perhaps we should detect any save error
				}
			}
			return true;
		}
		
		private function checkSchedule()
		{
			$db		= JFactory::getDbo();
			$query	= $db->getQuery(true);
			$date   = JFactory::getDate()->toSql();
			
			$query->select('*')
					->from($db->qn('#__cloudbackup_profiles'))
					->where('state = 1')
					->where(
							'('.
								$db->qn('lastattempt') . ' = 0' . ' OR '.
								'('. $db->qn('lastattempt') . ' + INTERVAL ' . $db->qn('interval') . ' HOUR <= ' . $db->quote($date) . ')' . ' OR '.
								'('. $db->qn('lastattempt') . ' + INTERVAL (' . $db->qn('interval') . ' * 30) MINUTE <= ' . $db->quote($date) . ' AND ' . $db->qn('attemptresult') . ' <> ' . $db->quote('success') . ')'.
							')'
						)
					->order($db->qn('lastattempt') . ' ASC');
					
			$db->setQuery($query, 0, 1);
			$profile = $db->loadObject();
			
			if($db->getErrorNum()) {
				$this->mailbody[] = 'Error: Failed to choose backup profile Schedule. Backup shall not be carried out.';
				return false;
			} elseif(empty($profile)) {
				return false;
			}
			$this->profile = $profile;
			return true;
		}
		
		private function markAction($finish = false)
		{
			$db		= JFactory::getDbo();
			$query	= $db->getQuery(true);
			$date   = JFactory::getDate()->toSql();
			
			$query->update($db->qn('#__cloudbackup_profiles'));
			if($finish) {
				$query->set($db->qn('attemptresult') . ' = '. $db->quote('success'));		// Failure should also be marked, TODO: Later
			} else {
				$query->set($db->qn('lastattempt')   . ' = '. $db->quote($date))
					  ->set($db->qn('attemptresult') . ' = '. $db->quote(''));
			}
			$query->where($db->qn('id') .' = '. $db->quote($this->profile->id));
			$db->setQuery($query);
			$db->query();
			
			if($db->getErrorNum()) {
				$this->mailbody[] = 'Error: Failed to '.( $finish ? 'complete' : 'initialize'). ' backup process.';
				return false;
			}
			$this->mailbody[] = 'Message: Backup process '.($finish ? 'completed' : 'initialized'). ' on '. $date;
			return true;
		}
		
		private function notifyAdmin()
		{
			$config = JFactory::getConfig();
			$data   = array(
						'fromname' => $config->get('fromname'),
						'mailfrom' => $config->get('mailfrom'),
						'sitename' => $config->get('sitename'),
						'siteurl'  => JUri::root(),
					);
			
			$subject = JText::sprintf('PLG_CLOUDBACKUP_EMAIL_SUBJECT', $data['sitename'], $data['siteurl']);
			$body    = JText::sprintf('PLG_CLOUDBACKUP_EMAIL_BODY_BEGIN', implode("\n\n", $this->mailbody));
			$db 	 = JFactory::getDbo();
			$query   = $db->getQuery(true);
			$query->select('name, email, sendEmail')
					->from('#__users')
					->where('sendEmail = 1');
			$db->setQuery($query);
			$rows = $db->loadObjectList();
			
			foreach($rows as $row) {
				$return = JFactory::getMailer()->sendMail($data['mailfrom'], $data['fromname'], $row->email, $subject, $body);
			}
		}
		
	}
	
	//// DEBUG PURPOSE /////////////////////////////////////////////
	function printr($x, $d=0, $v=0) {
		echo '<pre>';
			$v ? var_dump($x) : print_r($x);
		echo '</pre>';
		if($d) jexit();
	}
	//// DEBUG PURPOSE /////////////////////////////////////////////