<?php

/**
 * @package     Extly.Components
 * @subpackage  plgSystemKmAutoTweet - Post comments to AutoTweet
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweetNG for Komento-Plugin - AutoTweetNG Component is not installed or not enabled.');

	return;
}

require_once JPATH_ADMINISTRATOR . '/components/com_autotweet/helpers/autotweetbase.php';

/**
 * plgSystemKmAutoTweet class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PlgSystemKmAutoTweet extends PlgAutotweetBase
{
	const TYPE_FIRSTCOMMENT = 1;
	const TYPE_REPLY = 2;

	protected $onlyfirst = 0;

	protected $templatecomment = null;

	protected $templatereply = null;

	protected $use_text_count = null;

	/**
	 * Constructor.
	 *
	 * @param   string  &$subject  Param
	 * @param   array   $config    Configuration
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		$pluginParams = $this->params;

		if ($pluginParams)
		{
			$this->onlyfirst = $pluginParams->get('onlyfirst', 0);
			$this->templatecomment = $pluginParams->get('templatecomment', 'New comment: [title]!');
			$this->templatereply = $pluginParams->get('templatereply', 'New comment: [title]!');
			$this->use_text_count = $pluginParams->get('use_text_count75', SharingHelper::MAX_CHARS_TITLE);
		}
	}

	/**
	 * onAfterKomentoProcessNewComment
	 *
	 * @param   array  $data  New Comment
	 *
	 * @return	void
	 */
	public function onAfterKomentoProcessNewComment($data)
	{
		// Get object's owner id
		$owner = $data['comment']->created_by;

		// Check that this is first comment to the object
		$isfirst = ($data['comment']->lft == 1);

		$url = JRequest::getString('HTTP_REFERER', null, 'SERVER');

		if ($isfirst)
		{
			$typeinfo = self::TYPE_FIRSTCOMMENT;
		}
		else
		{
			$typeinfo = self::TYPE_REPLY;
		}

		$message = TextUtil::cleanText($data['comment']->comment);
		$data['message'] = $message;
		$this->postData($data['comment']->id, JFactory::getDate()->toSql(), $typeinfo, $message, $url, $data);
	}

	/**
	 * onAfterKomentoProcessNewComment
	 *
	 * @param   string  $component  Param
	 * @param   int     $cid        Param
	 * @param   array   $data       Param
	 *
	 * @return	void
	 */
	public function onAfterSaveComment($component, $cid, $data)
	{
		$this->onAfterKomentoProcessNewComment($data);
	}

	/**
	 * postData
	 *
	 * @param   integer  $commentid  Param
	 * @param   date     $date       Param
	 * @param   int      $typeinfo   Param
	 * @param   string   $message    Param
	 * @param   string   $url        Param
	 * @param   string   &$data      Param
	 *
	 * @return	void
	 */
	public function postData($commentid, $date, $typeinfo, $message, $url, &$data)
	{
		$image_url = null;

		if (array_key_exists('html', $data))
		{
			$image_url = $this->getImageFromText($data['html']);
		}

		if (array_key_exists('message', $data))
		{
			$image_url = TextUtil::getImageFromTextWithBrackets($data['message']);
		}

		// It is added in the general cleaning routine
		// $message = $this->cleanImagesWithBrackets($message);

		$native_object = json_encode($data);
		$this->postStatusMessage($commentid, $date, $message, $typeinfo, $url, $image_url, $native_object);
	}

	/**
	 * getExtendedData
	 *
	 * @param   string  $id              Param.
	 * @param   string  $typeinfo        Param.
	 * @param   string  &$native_object  Param.
	 *
	 * @return	array
	 */
	public function getExtendedData($id, $typeinfo, &$native_object)
	{
		if ($typeinfo == self::TYPE_FIRSTCOMMENT)
		{
			$template = $this->templatecomment;
		}
		else
		{
			$template = $this->templatereply;
		}

		$data = json_decode($native_object, true);
		$title = $data['message'];

		$message = $this->message;

		$message = $template;
		$message = str_replace("[title]", $message, $data['comment']);
		$message = $this->getMessagetext(0, $this->use_text_count, $message, $data['message']);

		$config = JFactory::getConfig();

		// It is added in the general cleaning routine
		// $title = $this->cleanImagesWithBrackets($title);	$message = $this->cleanImagesWithBrackets($message);

		// Return values
		$data = array(
			'title' => $title,
			'text' => $message,
			'fulltext' => $config->MetaTitle,
			'author' => $this->getAuthorUsername($comment['created_by']),
			'is_valid' => true
		);

		return $data;
	}
}
