<?php

/**
 * @version		$Id: metarobots.php 20196 2011-03-04 02:40:25Z mrichey $
 * @package		plg_system_metarobots
 * @copyright	Copyright (C) 2005 - 2011 Michael Richey. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

jimport('joomla.plugin.plugin');

class plgSystemMetaRobots extends JPlugin {

    /**
     * @param	JForm	$form	The form to be altered.
     * @param	array	$data	The associated data for the form.
     *
     * @return	boolean
     * @since	1.6
     */
    function onContentPrepareForm($form) {
        if (!($form instanceof JForm)) return false;        
        $validforms = array(
            'com_content.article' => array(
                'jform_metadata_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_menus.item' => array(
                'jform_params_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_categories.categorycom_content' => array(
                'jform_metadata_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_weblinks.weblink' => array(
                'jform_metadata_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_categories.categorycom_weblinks' => array(
                'jform_metadata_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_newsfeeds.newsfeed' => array(
                'jform_metadata_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_categories.categorycom_newsfeeds' => array(
                'jform_metadata_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_contact.contact' => array(
                'jform_metadata_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_categories.categorycom_contact' => array(
                'jform_metadata_robots' => array(
                    'group' => 'params',
                    'default' => true
                )
            ),
            'com_config.application' => array(
                'jform_robots' => array(
                    'group' => null,
                    'default' => false
                )
            ),
        );
        
        // Check we are manipulating a valid form.
        $formname = $form->getName();
        if (!in_array($formname, array_keys($validforms)))
            return true;

        // awesome, we're in business - lets get started
        JHtml::_('behavior.framework', true);
        JFactory::getLanguage()->load('plg_system_metarobots', JPATH_ADMINISTRATOR);
        $doc = JFactory::getDocument();
        $app = JFactory::getApplication();
        $globalrobots = (strlen(trim($app->getCfg('robots'))) == 0) ? array('index', 'follow') : explode(', ', trim($app->getCfg('robots')));
        $doc->addStyleSheet(JURI::root(true) . '/media/plg_system_metarobots/css/metarobots.css');
        $doc->addScript(JURI::root(true) . '/media/plg_system_metarobots/js/metarobots.js');

        // we will need the field id/name to continue - this returns only one
        foreach ($validforms[$formname] as $id => $opts) {
            $form->setFieldAttribute('robots', 'type', 'text', $opts['group']);
            $form->setFieldAttribute('robots', 'readonly', 'true', $opts['group']);
            $form->setFieldAttribute('robots', 'class', 'readonly', $opts['group']);
        }
        $options = array('id' => $id, 'allowdefault' => $opts['default'] ? true : false, 'global' => $globalrobots);
        $doc->addScriptDeclaration("\nwindow.plg_system_metarobots_options = " . json_encode($options) . ";\n");
        $doc->addStyleDeclaration('#' . $id . ' {display:none;}');
        return true;
    }
}